@@ -1,4 +1,4 @@
-package CTK; # $Id: CTK.pm 184 2015-05-12 10:15:56Z minus $
+package CTK; # $Id: CTK.pm 201 2017-05-02 10:37:04Z minus $
 use Moose; #use strict;
 
 =head1 NAME
@@ -7,11 +7,7 @@ CTK - Command-line ToolKit
 
 =head1 VERSION
 
-Version 1.17
-
-=head1 REVISION
-
-$Revision: 184 $
+Version 1.18
 
 =head1 SYNOPSIS
 
@@ -19,12 +15,11 @@ $Revision: 184 $
     use CTK qw( :BASE ); # :SUBS and :VARS tags to export
     use CTK qw( :SUBS ); # :SUBS tag only to export
     use CTK qw( :VARS ); # :VARS tag only to export
-    
+
     my $c = new CTK;
     my $c = new CTK (
         prefix       => 'myprogram',
         suffix       => 'sample',
-        syspaths     => 0, # need use system paths
         cfgfile      => '/path/to/conf/file.conf',
         voidfile     => '/path/to/void/file.txt',
         needconfig   => 1, # need creating empty config file
@@ -39,9 +34,107 @@ CTKlib - Command-line ToolKit library (CTKlib). Command line interface (CLI)
 
 =head1 DESCRIPTION
 
-Sorry. Detail manual is preparing now and it will be available later
+CTKlib - is library that provides "extended-features" (utilities) for your robots written on Perl.
+Most of the functions and methods this module written very simple language and easy to understand.
+To work with CTKlib, you just need to start using it!
 
-See C<README> file
+See also C<README> file
+
+=head2 new
+
+    my $c = new CTK;
+
+    my $c = new CTK ( syspaths => 1 ); # need use system paths
+
+    my $c = new CTK (
+        prefix       => 'myprogram',
+        suffix       => 'sample',
+        cfgfile      => '/path/to/conf/file.conf',
+        voidfile     => '/path/to/void/file.txt',
+        needconfig   => 1, # need creating empty config file
+        loglevel     => 'info', # or '1'
+        logfile      => CTK::catfile($LOGDIR,'foo.log'),
+        logseparator => ' ', # as default
+    );
+
+Main constructor. All the params are optional
+
+=over 8
+
+=item B<cfgfile>
+
+Full path to the configuration file of the your project
+
+=item B<logfile>
+
+Full path to the log file
+
+=item B<loglevel>
+
+Logging level. It can be set as: debug, info, notice, warning, error, crit, alert, emerg, fatal and except.
+
+=item B<logseparator>
+
+Separator for log columns. The default is the space character (" ")
+
+=item B<needconfig>
+
+Specifies the need to create an empty configuration file. Not recommended for use
+
+=item B<prefix>, B<suffix>
+
+Prefix and suffix of the name your project
+
+=item B<syspaths>
+
+The directive specifies the use of system paths for configuration, logging and allocation of temporary and working data
+
+=item B<voidfile>
+
+Full path to the VOID file for connections testing
+
+=back
+
+=head2 init, again
+
+For internal use only. Please not call this functions
+
+=head2 debug
+
+Prints debug information on STDOUT or into general log-file
+
+=head2 debugmode
+
+Returns debug flag set. 1 - on, 0 - off
+
+=head2 exception
+
+Shows error and immediately exit from program with die state
+
+=head2 logmode
+
+Returns log flag set. 1 - on, 0 - off
+
+=head2 say
+
+Prints a string or a list of strings implicitly appends a newline
+
+=head2 silentmode
+
+Returns the verbose flag in the opposite value. 0 - verbose, 1 - silent.
+See L<"/verbosemode">
+
+=head2 testmode
+
+Returns test flag set. 1 - on, 0 - off
+
+=head2 tms
+
+Returns timestamp. For example: [8588] {TimeStamp: +0.5580 sec}
+
+=head2 verbosemode
+
+Returns verbose flag set. 1 - on, 0 - off
 
 =head1 HISTORY
 
@@ -109,11 +202,11 @@ The usual warnings if it can't read or write the files involved.
 
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://www.serzik.ru> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -127,27 +220,29 @@ See C<LICENSE> file
 
 use vars qw/
         $VERSION
-        $TM $EXEDIR $DATADIR $CONFDIR $CONFFILE $LOGDIR $LOGFILE %ARGS %OPT @OPTSYSDEF
+        $TM $EXEDIR $DATADIR $CONFDIR $CONFFILE $LOGDIR $LOGFILE %ARGS %OPT @OPTSYSDEF $TERMCHSET
     /;
-$VERSION = '1.17';
+$VERSION = '1.18';
 
 use constant {
     DEBUG     => 1, # 0 - off, 1 - on, 2 - all (+ http headers and other)
     LOG       => 1, # 0 - off, 1 - on
     TESTMODE  => 1, # 0 - off, 1 - on (         )
+    VERBOSE   => 1, # 0 - off, 1 - on
+    SILENT    => 1, # 0 - off, 1 - on (silent/quiet)
 
     WIN       => $^O =~ /mswin/i ? 1 : 0,
     NULL      => $^O =~ /mswin/i ? 'NUL' : '/dev/null',
     TONULL    => $^O =~ /mswin/i ? '>NUL 2>&1' : '>/dev/null 2>&1',
     ERR2OUT   => '2>&1',
-    
-    TERMCHSET => 'utf8', #    
-    
+
+    TERMCHSETD=> 'utf8', #    
+
     LOGFILED  => 'ctklib.log',    #    
     CFGFILED  => 'ctklib.conf',   #    
     CFGFILE   => '[PREFIX].conf', #  
     VOIDFILE  => 'void.txt',      #  VOID (    )
-    
+
     DATADIRD  => 'data', #     
     CONFDIRD  => 'conf', #     
     LOGDIRD   => 'log',  #     
@@ -155,20 +250,22 @@ use constant {
 
 use base qw /Exporter/; # extends qw/CTK::Arc/;
 our @EXPORT = qw(
-        say debug tms exception testmode debugmode logmode
+        say debug tms exception testmode debugmode logmode verbosemode silentmode
         $EXEDIR $DATADIR $CONFDIR $CONFFILE $LOGDIR $LOGFILE %OPT @OPTSYSDEF
     );
 our @EXPORT_OK = qw(
-        say debug tms exception testmode debugmode logmode
+        say debug tms exception testmode debugmode logmode verbosemode silentmode
         $TM $EXEDIR $DATADIR $CONFDIR $CONFFILE $LOGDIR $LOGFILE %OPT @OPTSYSDEF
     );
 our %EXPORT_TAGS = (
-        ALL     => [qw($TM $EXEDIR $DATADIR $CONFDIR $CONFFILE $LOGDIR $LOGFILE %OPT @OPTSYSDEF say debug tms exception testmode debugmode logmode)],
-        BASE    => [qw($EXEDIR $DATADIR $CONFDIR $LOGDIR say debug tms exception testmode debugmode logmode)],
-        FUNC    => [qw(say debug tms exception testmode debugmode logmode)],
-        FUNCS   => [qw(say debug tms exception testmode debugmode logmode)],
-        SUB     => [qw(say debug tms exception testmode debugmode logmode)],
-        SUBS    => [qw(say debug tms exception testmode debugmode logmode)],
+        ALL     => [qw($TM $EXEDIR $DATADIR $CONFDIR $CONFFILE $LOGDIR $LOGFILE %OPT @OPTSYSDEF
+                       say debug tms exception testmode debugmode logmode verbosemode silentmode)],
+        BASE    => [qw($EXEDIR $DATADIR $CONFDIR $LOGDIR say debug tms exception testmode debugmode
+                       logmode verbosemode silentmode)],
+        FUNC    => [qw(say debug tms exception testmode debugmode logmode verbosemode silentmode)],
+        FUNCS   => [qw(say debug tms exception testmode debugmode logmode verbosemode silentmode)],
+        SUB     => [qw(say debug tms exception testmode debugmode logmode verbosemode silentmode)],
+        SUBS    => [qw(say debug tms exception testmode debugmode logmode verbosemode silentmode)],
         VARS    => [qw($TM $EXEDIR $DATADIR $CONFDIR $CONFFILE $LOGDIR $LOGFILE %OPT @OPTSYSDEF)],
         NONE    => [qw()],
     );
@@ -181,7 +278,7 @@ use CTK::CPX;
 use CTK::Util;
 
 ########################
-##  
+## Init functions
 ########################
 sub init {
     # GLOBAL VARS
@@ -196,71 +293,78 @@ sub init {
         'debug'     => DEBUG    ? 0 : 1,    #        "!"
         'log'       => LOG      ? 0 : 1,    #        "!"
         'testmode'  => TESTMODE ? 0 : 1,    #        "!"
+        'verbose'   => 0,
     );
     @OPTSYSDEF = ( #   .   : humvdlcyt?
         #  
-        "help|usage|h|u|?",                 #   
+        "help|usage|h|?",                   #   
         "man|m",                            # 
         "version|ver|v",                    #  
-    
+
         #  
         "debug|d",                          #  --  ,   . DEBUG
         "log|l",                            #  --  ,   . LOG
-        "logclear|logclean|clean|clear|c",  #     
-        "signature|sign|msg|y=s",           #   
-    
+        "logclear|logclean|c",              #     
+        "signature|sign|y=s",               #   
+
         #  
         "testmode|test|t",                  #    --   . TESTMODE
     );
-}    
+}
 BEGIN { init() }
 *again = \&init;
 
-#    
+#   
 if (WIN) {
-    tie *CTKCP, 'CTK::CPX', 'cp866'
+    my $tcs = $TERMCHSET || 'cp866';
+    tie *CTKCP, 'CTK::CPX', $tcs;
 } else {
-    if (TERMCHSET) { tie *CTKCP, 'CTK::CPX', TERMCHSET } else { *CTKCP = *STDOUT }
-}    
+    my $tcs = $TERMCHSET // TERMCHSETD;
+    if ($tcs) { tie *CTKCP, 'CTK::CPX', $tcs } else { *CTKCP = *STDOUT }
+}
 
 ########################
-##  
+## Base functions
 ########################
 sub say { print CTKCP @_ ? @_ : '',"\n"}
-sub debug { 
+sub debug {
     unshift(@_,$OPT{signature}." ") if defined $OPT{signature};
     if (LOG && $OPT{'log'}) {
         my @dt=localtime(time());
         if (open(FD, ">>", $LOGFILE)) {
             flock FD, 2 or carp("Can't lock file: $!");
-            print FD sprintf("[%02d.%02d.%04d %02d:%02d:%02d] ",$dt[3],$dt[4]+1,$dt[5]+1900,$dt[2],$dt[1],$dt[0]), @_ ? @_ : '', "\n"; 
+            print FD sprintf("[%02d.%02d.%04d %02d:%02d:%02d] ",$dt[3],$dt[4]+1,$dt[5]+1900,$dt[2],$dt[1],$dt[0]), @_ ? @_ : '', "\n";
             close(FD);
         } else {
-            carp("Can't open file to write: $!"); 
+            carp("Can't open file to write: $!");
         }
     }
     return 1 unless DEBUG && $OPT{debug};
     say(@_);
 }
 sub tms { "[$$] {TimeStamp: ".sprintf("%+.*f",4, gettimeofday()-$TM)." sec}" }
-sub exception { 
-    my $clr = " [ CALLER: ".join("; ", caller())." ]"; 
-    debug(@_,$clr); 
+sub exception {
+    my $clr = " [ CALLER: ".join("; ", caller())." ]";
+    debug(@_,$clr);
     confess(translate(join("",(@_,$clr))));
 }
-sub testmode { return CTK::TESTMODE && $OPT{testmode} } #   TESTMODE  !
-sub debugmode { return (CTK::DEBUG && $OPT{debug}) ? DEBUG : undef } #   DEBUG!
-sub logmode { return CTK::LOG && $OPT{'log'} } #   LOG  !
+
+# Modes
+sub testmode { return CTK::TESTMODE && $OPT{testmode} }
+sub debugmode { return (CTK::DEBUG && $OPT{debug}) ? DEBUG : undef }
+sub logmode { return CTK::LOG && $OPT{'log'} }
+sub verbosemode { return CTK::VERBOSE && $OPT{verbose} }
+sub silentmode { return CTK::SILENT && !$OPT{verbose} }
 
 ########################
-##   Moose
+## General Moose Methods
 ########################
 
 with 'CTK::CLI' => {
             -excludes => [qw/_cli_select/],
         },
-     'CTK::File' => { 
-            -excludes => [qw/_error/], 
+     'CTK::File' => {
+            -excludes => [qw/_error _expand_wildcards/],
         },
      'CTK::Crypt' => {},
      'CTK::Arc' => {
@@ -270,44 +374,44 @@ with 'CTK::CLI' => {
             -alias    => {
                     _debug_http => 'debug_http',
                 },
-            -excludes => [qw/_error/], 
+            -excludes => [qw/_error/],
         },
-     'CTK::Log' => { 
-            -excludes => [qw/_flush/], 
+     'CTK::Log' => {
+            -excludes => [qw/_flush/],
         };
 
 has 'revision'  => ( # 
         is      => 'ro',
         isa     => 'Str',
-        default => q/$Revision: 184 $/ =~ /(\d+\.?\d*)/ ? $1 : '0',
+        default => q/$Revision: 201 $/ =~ /(\d+\.?\d*)/ ? $1 : '0',
         lazy    => 1,
         init_arg=> undef,
     );
 has 'script'    => ( #  
-        is      => 'ro', 
-        isa     => 'Str', 
+        is      => 'ro',
+        isa     => 'Str',
         default => $Script,
     );
 has 'prefix'    => ( #  (     CTKlib)
-        is      => 'rw', 
-        isa     => 'Str', 
+        is      => 'rw',
+        isa     => 'Str',
         default => ($Script =~ /^(.+?)\./ ? $1 : $Script),
     );
 has 'suffix'    => ( #  (     CTKlib)
-        is      => 'rw', 
-        isa     => 'Str', 
+        is      => 'rw',
+        isa     => 'Str',
         default => '',
     );
 has 'cfgfile'   => ( #    
-        is      => 'rw', 
-        isa     => 'Str', 
+        is      => 'rw',
+        isa     => 'Str',
         default => CFGFILE,
         trigger => sub {
                 my $self = shift;
                 my $val = shift || '';
                 my $old_val = shift || '';
                 #debug "TRIGGER: $self, $val, $old_val";
-                $self->{cfgfile} = dformat($val,{ 
+                $self->{cfgfile} = dformat($val,{
                         PREFIX   => $self->prefix(),
                         SUFFIX   => $self->suffix(),
                         EXT      => 'conf',
@@ -317,15 +421,15 @@ has 'cfgfile'   => ( #
             },
     );
 has 'voidfile'  => ( #     ,   
-        is      => 'rw', 
-        isa     => 'Str', 
+        is      => 'rw',
+        isa     => 'Str',
         default => VOIDFILE,
         trigger => sub {
                 my $self = shift;
                 my $val = shift || '';
                 my $old_val = shift || '';
                 # debug "TRIGGER: $self, $val, $old_val";
-                $self->{voidfile} = dformat($val,{ 
+                $self->{voidfile} = dformat($val,{
                         PREFIX   => $self->prefix(),
                         SUFFIX   => $self->suffix(),
                         EXT      => 'txt',
@@ -334,33 +438,33 @@ has 'voidfile'  => ( #     ,
             },
     );
 has 'config'    => ( #   (Config::General)
-        is      => 'rw', 
+        is      => 'rw',
         isa     => 'HashRef',
-    ); 
+    );
 has 'options'   => ( #     (Getopt::Long)
-        is      => 'rw', 
-        isa     => 'HashRef', 
+        is      => 'rw',
+        isa     => 'HashRef',
         default => sub { \%OPT } ,
-    ); 
+    );
 has 'needconfig'=> ( #         ?
-        is      => 'rw', 
-        isa     => 'Bool', 
+        is      => 'rw',
+        isa     => 'Bool',
         default => 0,
-    ); 
+    );
 has 'syspaths'=> ( #      ,  ""?
-        is      => 'rw', 
-        isa     => 'Bool', 
+        is      => 'rw',
+        isa     => 'Bool',
         default => 0,
-    ); 
+    );
 has 'exedir'    => ( #      
-        is      => 'ro', 
-        isa     => 'Str', 
+        is      => 'ro',
+        isa     => 'Str',
         default => sub { $EXEDIR },
         lazy    => 1,
     );
 has 'datadir'   => ( #      
-        is      => 'rw', 
-        isa     => 'Str', 
+        is      => 'rw',
+        isa     => 'Str',
         default => sub { $DATADIR },
         lazy    => 1,
         trigger => sub {
@@ -369,11 +473,11 @@ has 'datadir'   => ( #
                 # debug "TRIGGER: $self, $val";
                 $DATADIR = $val;
             },
-        
+
     );
 has 'confdir'   => ( #       
-        is      => 'rw', 
-        isa     => 'Str', 
+        is      => 'rw',
+        isa     => 'Str',
         default => sub { $CONFDIR },
         lazy    => 1,
         trigger => sub {
@@ -385,8 +489,8 @@ has 'confdir'   => ( #
 
     );
 has 'logdir'    => ( #      
-        is      => 'rw', 
-        isa     => 'Str', 
+        is      => 'rw',
+        isa     => 'Str',
         default => sub { $LOGDIR },
         lazy    => 1,
         trigger => sub {
@@ -398,15 +502,15 @@ has 'logdir'    => ( #
             },
 
     );
-    
+
 sub BUILD { # new
     my $self = shift;
     my $options = shift || {};
-    
+
     #  -
     my $prefix = $self->prefix();
     my $suffix = $self->suffix();
-    
+
     #          
     my $oldcfgfile = $self->cfgfile();
     if ($self->syspaths()) {
@@ -425,16 +529,16 @@ sub BUILD { # new
     # Paths rebuilding
     $self->cfgfile($oldcfgfile); # CFGFILE rebuilding
     $self->voidfile($self->voidfile()); # VOIDFILE rebuilding
-    
+
     #   
     $self->config({_loadconfig($self->cfgfile(), $self->needconfig())});
-        
+
     #debug Dumper(\@_);
     return 1;
 };
 sub AUTOLOAD {
     #          
-    #     ,     
+    #     ,    
     my $self = shift;
     our $AUTOLOAD;
     my $AL = $AUTOLOAD;
@@ -452,7 +556,7 @@ sub AUTOLOAD {
     } else {
         exception("Can't find procedure \"$AL\"!");
     }
-    return;    
+    return;
 }
 sub DEMOLISH { # DESTROY
     #  
@@ -468,11 +572,11 @@ sub _loadconfig {
 
     my %config = (loadstatus => 0);
     my $conf;
-    
+
     #    
     if ($cfile && -e $cfile) {
-        $conf = new Config::General( 
-                -ConfigFile         => $cfile, 
+        $conf = new Config::General(
+                -ConfigFile         => $cfile,
                 -ConfigPath         => [$EXEDIR, $CONFDIR],
                 -ApacheCompatible   => 1,
                 -LowerCaseNames     => 1,
@@ -482,10 +586,10 @@ sub _loadconfig {
         $config{configfiles} = [$conf->files];
         $config{loadstatus} = 1;
     }
-    
+
     #     
     return %config unless $need;
-    
+
     #    ,       ()
     unless (%config && $config{loadstatus}) {
         debug "Configuration save into \"$cfile\"...";
@@ -496,35 +600,6 @@ sub _loadconfig {
     return %config;
 }
 
-#
-##    ,      --     
-##     
-##
-sub getObj {
-    my $self = shift;
-    return $self;
-}
-sub getMeta { #       
-    my $meta = __PACKAGE__->meta();
- 
-    debug "Attributes:";
-    for my $attribute ( $meta->get_all_attributes ) {
-        debug " ", $attribute->name();
- 
-        if ( $attribute->has_type_constraint ) {
-            debug "  type: ", $attribute->type_constraint->name;
-        }
-    }
- 
-    debug;
-    debug "Methodts:";
-    for my $method ( sort {$a->package_name().$a->name() cmp $b->package_name().$b->name()} $meta->get_all_methods ) {
-        debug(" ", $method->package_name, " :: ", $method->name);
-    }
-    
-    return $meta;
-}
-
 no Moose;
 __PACKAGE__->meta->make_immutable;
 1;
