@@ -1,4 +1,4 @@
-package CTK::Arc; # $Id: Arc.pm 180 2014-04-14 19:59:32Z minus $
+package CTK::Arc; # $Id: Arc.pm 193 2017-04-29 07:30:55Z minus $
 use Moose::Role; # use Data::Dumper; $Data::Dumper::Deparse = 1;
 
 =head1 NAME
@@ -7,11 +7,7 @@ CTK::Arc - Archives working
 
 =head1 VERSION
 
-Version 1.70
-
-=head1 REVISION
-
-$Revision: 180 $
+Version 1.71
 
 =head1 SYNOPSIS
 
@@ -23,7 +19,7 @@ $Revision: 180 $
             -list   => qr/rar/, # Source mask (regular expression, filename or ArrayRef of files)
             -arcdef => $config->{arc}, # Archive attributes (Hashref)
         );
-        
+
     # Internal extracting
     $c->fextract(
             -in     => CTK::catfile($CTK::DATADIR,'in'),  # Source directory (archives)
@@ -31,11 +27,9 @@ $Revision: 180 $
             -method => 'zip', # Zip archive
             -list   => qr/zip/, # Source mask (regular expression, filename or ArrayRef of files)
             -arcdef => $config->{arc}, # Archive attributes (Hashref)
-        );        
-        
-    
-    
-    # External files compressing 
+        );
+
+    # External files compressing
     $c->fcompress(
             -in     => CTK::catfile($CTK::DATADIR,'in'), # Source directory (files)
             -out    => CTK::catfile($CTK::DATADIR,'out','ttt.rar'), # Archive name (filename)
@@ -47,9 +41,9 @@ $Revision: 180 $
 
 Sample of $config->{arc} records:
 
-    ARC => { 
+    ARC => {
         tgz   =>  {
-            "type"       => "tar", # name 
+            "type"       => "tar", # name
             "ext"        => "tgz", # extension
             "create"     => "tar -zcpf [FILE] [LIST]", # create command
             "extract"    => "tar -zxpf [FILE] [DIRDST]", # extract command
@@ -98,13 +92,49 @@ Reserved
 
 =back
 
+=head2 METHODS
+
+=head3 fextract
+
+    # External extracting
+    $c->fextract(
+            -in     => CTK::catfile($CTK::DATADIR,'in'),  # Source directory (archives)
+            -out    => CTK::catfile($CTK::DATADIR,'out'), # Destination directory (files)
+            -method => 'ext',
+            -list   => qr/rar/, # Source mask (regular expression, filename or ArrayRef of files)
+            -arcdef => $config->{arc}, # Archive attributes (Hashref)
+        );
+
+    # Internal extracting
+    $c->fextract(
+            -in     => CTK::catfile($CTK::DATADIR,'in'),  # Source directory (archives)
+            -out    => CTK::catfile($CTK::DATADIR,'out'), # Destination directory (files)
+            -method => 'zip', # Zip archive
+            -list   => qr/zip/, # Source mask (regular expression, filename or ArrayRef of files)
+            -arcdef => $config->{arc}, # Archive attributes (Hashref)
+        );
+
+Extracting the files group
+
+=head3 fcompress
+
+    # External files compressing
+    $c->fcompress(
+            -in     => CTK::catfile($CTK::DATADIR,'in'), # Source directory (files)
+            -out    => CTK::catfile($CTK::DATADIR,'out','ttt.rar'), # Archive name (filename)
+            -list   => qr//, # Source mask (regular expression, filename or ArrayRef of files)
+            -arcdef => $config->{arc}, # Archive attributes (Hashref)
+        );
+
+Compressing the files group
+
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2014 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -117,7 +147,7 @@ See C<LICENSE> file
 =cut
 
 use constant {
-    ARC       => { 
+    ARC       => {
         tgz   =>  {
             "type"       => "tar",
             "ext"        => "tgz",
@@ -136,16 +166,16 @@ use constant {
             "list"       => "tar -ztf [FILE]",
             "nocompress" => "tar -cpf [FILE]"
         },
-        zip   => {    
+        zip   => {
             "type"       => "zip",
             "ext"        => "zip",
             "create"     => $^O =~ /mswin/i ? "zip -rqq [FILE] [LIST]" : "zip -rqqy [FILE] [LIST]",
-            "extract"    => $^O =~ /mswin/i ? "unzip -uqqoX [FILE] -d [DIRDST]" : "unzip -uqqoX [FILE] [DIRDST]", 
+            "extract"    => $^O =~ /mswin/i ? "unzip -uqqoX [FILE] -d [DIRDST]" : "unzip -uqqoX [FILE] [DIRDST]",
             "exclude"    => "-x\@",
             "list"       => "unzip -lqq",
             "nocompress" => "zip -qq0"
         },
-        rar   => { 
+        rar   => {
             "type"       => "rar",
             "ext"        => "rar",
             "create"     => $^O =~ /mswin/i ? "rar a -r -y [FILE] [LIST]" : "rar a -r -ol -y [FILE] [LIST]",
@@ -158,7 +188,7 @@ use constant {
 };
 
 use vars qw/$VERSION/;
-$VERSION = '1.70';
+$VERSION = '1.71';
 
 use CTK::Util qw(:API :FORMAT :ATOM);
 use Archive::Tar;
@@ -168,26 +198,26 @@ use Archive::Extract;
 sub fextract {
     #     
     my $self; $self = shift if (@_ && $_[0] && ref($_[0]) eq 'CTK');
-    
+
     my @args = @_;
     my ($method, $dirin, $dirout, $listmsk, $arcdef);
-       ($method, $dirin, $dirout, $listmsk, $arcdef) = 
+       ($method, $dirin, $dirout, $listmsk, $arcdef) =
             read_attributes([
                 ['METHOD','METH','TYPE'],
                 ['DIRIN','IN','INPUT','DIRSRC','SRC'],
                 ['DIROUT','OUT','OUTPUT','DIRDST','DST'],
                 ['LISTMSK','LIST','MASK','LST','MSK','FILE','FILES'],
                 ['ARC','ARCDEF','ARCSET','SET','DEF'],
-                
+
             ],@args) if defined $args[0];
-    
+
     $method   ||= 'ext'; #    zip / tar / ext
     $dirin    ||= ''; #  
     $dirout   ||= ''; #  
     $listmsk  ||= ''; #       
     $arcdef   ||= ''; #  (  )     arc (  )
     my $list;
-    
+
     if (ref($listmsk) eq 'ARRAY') {
         # 
         $list = $listmsk;
@@ -214,7 +244,7 @@ sub fextract {
                 #CTK::debug("   --- Extracting \"$fan\"...");
                 $tar->extract_file( $fan, $dirout ? catfile($dirout,$fan) : $fan );
             }
-        
+
         }
     } elsif ($method eq 'zip') {
         #CTK::debug(" ZIP-  \"$dirin\"   \"$dirout\"...");
@@ -263,8 +293,8 @@ sub fextract {
                 #CTK::debug("   --- ERROR:      : $fn");
                 carp("EXTERNAL EXTRACTION ERROR: Unknow format $fn"); #unless CTK::debugmode();
             }
-        }    
-            
+        }
+
     } else {
         #CTK::debug("ERROR:   () : $method");
         carp("ERROR: Unknow archive's format or method: $method"); #unless CTK::debugmode();
@@ -273,26 +303,26 @@ sub fextract {
 sub fcompress {
     #     
     my $self; $self = shift if (@_ && $_[0] && ref($_[0]) eq 'CTK');
-    
+
     my @args = @_;
     my ($dirin, $fout, $listmsk, $arcdef);
-       ($dirin, $fout, $listmsk, $arcdef) = 
+       ($dirin, $fout, $listmsk, $arcdef) =
             read_attributes([
                 ['DIRIN','IN','INPUT','DIRSRC','SRC'],
                 ['FILEOUT','OUT','OUTPUT','FILEDST','DST','FOUT','NAME','ARCHIVE'],
                 ['LISTMSK','LIST','MASK','LST','MSK','FILE','FILES'],
                 ['ARC','ARCDEF','ARCSET','SET','DEF'],
-                
+
             ],@args) if defined $args[0];
 
-        
+
     $dirin    ||= ''; #   (  !!! -   )
     $fout     ||= ''; #    ( )
     $listmsk  ||= ''; #          
     $arcdef   ||= ''; #  (  )     arc (  )
     my $list;
     my $dlist;
-    
+
     if (ref($listmsk) eq 'ARRAY') {
         # 
         $list = $listmsk;
@@ -307,10 +337,10 @@ sub fcompress {
     }
     $list   ||= [];
     $dlist  ||= [];
-    
-    
+
+
     #         ::debug(join "; ", @$list);
-    
+
     #
     # :
     # $arc{create} $bfile ".join(" ",@dirlist)." $exclude ";
@@ -330,7 +360,7 @@ sub fcompress {
             ARCDEF   => $arcdef,
         );
     #use Data::Dumper; ::debug(Dumper($arc)); return 1;
-        
+
     if ($arc) {
         my @C;
         push @C, $arc->{create};
@@ -338,7 +368,7 @@ sub fcompress {
     } else {
         #CTK::debug("   ERROR:      : $fout");
         carp("EXTERNAL COMPRESSING ERROR: Format undefined by $fout"); # unless CTK::debugmode();
-    }    
+    }
 }
 sub _getarc {
     #         undef
@@ -346,8 +376,8 @@ sub _getarc {
     my %dn = @_;
 	my $file = $dn{FILE} || '';
     my $def  = $dn{ARCDEF} || '';
-	
-    my $ext = ''; 
+
+    my $ext = '';
 	$ext = $1 if ($file && $file =~ /\.(\w+)$/);
 
     my $sec;
@@ -356,23 +386,21 @@ sub _getarc {
     } else {
         $sec = ARC->{$ext} || undef;
     }
-    
+
     unless( $sec ) {
         #CTK::debug( "Error: unknown archive format of file \"$file\": $ext" );
         carp("ERROR: Unknown archive format of file \"$file\": $ext"); # unless CTK::debugmode();
         return undef;
     }
     my %arc = %$sec;
-    
+
     #      
     foreach (values %arc) {
         $_ =~ s/\[(.+?)\]/($dn{uc($1)} || '')/eg;
     }
-    
+
 	return {%arc}; #    
 }
 
-#no Moose;
-#__PACKAGE__->meta->make_immutable;
 1;
 __END__
