@@ -1,4 +1,4 @@
-package CTK::CLI; # $Id: CLI.pm 180 2014-04-14 19:59:32Z minus $
+package CTK::CLI; # $Id: CLI.pm 193 2017-04-29 07:30:55Z minus $
 use Moose::Role; # use Data::Dumper; $Data::Dumper::Deparse = 1;
 
 =head1 NAME
@@ -7,11 +7,7 @@ CTK::CLI - Command line interface
 
 =head1 VERSION
 
-Version 1.70
-
-=head1 REVISION
-
-$Revision: 180 $
+Version 1.71
 
 =head1 SYNOPSIS
 
@@ -23,7 +19,7 @@ $Revision: 180 $
 
     my $v = $c->cli_select('Your select:',[qw/foo bar baz/],'bar');
     debug( "Your select: $v" );
-    
+
     my $v = $c->cli_select3('Your select:',[qw/foo bar baz/]);
     debug( "Your select: $v" );
 
@@ -31,10 +27,32 @@ $Revision: 180 $
 
 Command line interface. Prompt and select methods
 
+=head2 cli_prompt, cli_prompt3
+
+    my $v = $c->cli_prompt('Your name:', 'anonymous');
+    debug( "Your name: $v" );
+
+    my $v = $c->cli_prompt3('Your name:');
+    debug( "Your name: $v" );
+
+Show prompt string for typing data.
+The second method asks for data entry three times in case of incorrect attempts
+
+=head2 cli_select, cli_select3
+
+    my $v = $c->cli_select('Your select:',[qw/foo bar baz/],'bar');
+    debug( "Your select: $v" );
+
+    my $v = $c->cli_select3('Your select:',[qw/foo bar baz/]);
+    debug( "Your select: $v" );
+
+Show prompt string for select item.
+The second method asks for data entry three times in case of incorrect attempts
+
 =head1 TO DO
 
   * Use Term::ReadLine module. Sample:
-    
+
     BEGIN { $ENV{TERM} = "dumb" if $^O eq "MSWin32" }
     use Term::ReadLine ();
     use Text::ParseWords qw(shellwords);
@@ -54,11 +72,11 @@ Command line interface. Prompt and select methods
 
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2014 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -70,9 +88,8 @@ See C<LICENSE> file
 
 =cut
 
-
 use vars qw/$VERSION/;
-$VERSION = '1.70';
+$VERSION = '1.71';
 
 use CTK::Util qw(:API);
 use ExtUtils::MakeMaker qw/prompt/;
@@ -90,14 +107,14 @@ sub cli_prompt3 {
     #   cli_prompt 3    
     my $self; $self = shift if (@_ && $_[0] && ref($_[0]) eq 'CTK');
     my $msg = shift;
-    my $v = ''; 
-    my $i = 0; 
+    my $v = '';
+    my $i = 0;
     while ($i < 3) {$i++;
         $v = prompt($msg);
         last if defined($v) && $v ne '';
         CTK::say();
     }
-    return $v;    
+    return $v;
 }
 sub cli_select {
     #   
@@ -111,7 +128,7 @@ sub cli_select {
     CTK::say($v->[1]) if $v->[0];
     $v = cli_prompt(defined($msg) ? $msg : '', $d);
     $v = _cli_select($sel, $v);
-    
+
     return $v->[0] ? '' : $v->[1];
 }
 sub cli_select3 {
@@ -120,12 +137,12 @@ sub cli_select3 {
     my $msg = shift;
     my $sel = shift || [];
     my $def = shift;
-    
+
     unless ($sel && (!ref($sel) || ref($sel) eq 'ARRAY')) {
         carp("cli_select3: Call syntax error");
         $sel = [];
     }
-    
+
     my $v = _cli_select($sel);
     my $d = defined($def) ? $def : $v->[1];
     my $i = 0;
@@ -148,7 +165,7 @@ sub _cli_select {
         if (ref $v eq 'ARRAY') {
             if (defined($sel) && ($sel =~ /^\d+$/) && exists($v->[$sel-1])) {
                 return [0,$v->[$sel-1]];
-            } elsif (defined($sel) && grep {$_ eq $sel} @$v) {                
+            } elsif (defined($sel) && grep {$_ eq $sel} @$v) {
                 return [0,$sel];
             } else {
                 my $c=0;
@@ -164,10 +181,8 @@ sub _cli_select {
     }
 }
 
-#no Moose;
-#__PACKAGE__->meta->make_immutable;
 1;
 __END__
 
-    
+
 
