@@ -1,4 +1,4 @@
-package CTK::CPX; # $Id: CPX.pm 132 2013-03-12 14:45:50Z minus $
+package CTK::CPX; # $Id: CPX.pm 191 2017-04-28 18:29:58Z minus $
 use Moose;
 =head1 NAME
 
@@ -6,11 +6,7 @@ CTK::CPX - Converter between windows-1251 and your terminal encoding
 
 =head1 VERSION
 
-Version 1.01
-
-=head1 REVISION
-
-$Revision: 132 $
+Version 1.02
 
 =head1 SYNOPSIS
 
@@ -34,11 +30,11 @@ The usual warnings if it can't read or write the files involved.
 
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://serzik.ru> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -52,11 +48,19 @@ See C<LICENSE> file
 use namespace::autoclean;
 extends qw/Tie::Handle/;
 use Encode;
-our $VERSION = '1.01';
-sub TIEHANDLE { shift; my $incp = shift || 'cp866'; return bless [$incp], __PACKAGE__ }
+our $VERSION = '1.02';
+sub TIEHANDLE {
+    shift;
+    my $incp = shift || 'cp866';
+    return bless [$incp], __PACKAGE__;
+}
 sub PRINT {
     my $self = shift;
-    print STDOUT Encode::encode(($self->[0] || 'cp866'),Encode::decode('Windows-1251',join("",@_)));
+    my $cp = $self->[0] || 'cp866';
+    for (@_) {
+        print(STDOUT Encode::encode($cp, Encode::decode('Windows-1251',$_))) if defined;
+    }
+    1;
 }
 no Moose;
 # Force constructor inlining
