@@ -1,4 +1,4 @@
-package CTK::Helper; # $Id: Helper.pm 180 2014-04-14 19:59:32Z minus $
+package CTK::Helper; # $Id: Helper.pm 193 2017-04-29 07:30:55Z minus $
 use strict;
 
 =head1 NAME
@@ -7,11 +7,7 @@ CTK::Helper - Helper for building CTK scripts
 
 =head1 VIRSION
 
-Version 2.67
-
-=head1 REVISION
-
-$Revision: 180 $
+Version 2.68
 
 =head1 HISTORY
 
@@ -40,34 +36,56 @@ Added new types of projects
     use CTK;
     use CTKx;
     use CTK::Helper;
-    
+
     my $c = new CTK( syspaths => 1 );
     my $ctkx = CTKx->instance( c => $c );
-    
+
     my $h = new CTK::Helper (
         -type           => TYPE,
         -projectname    => PROJECTNAME,
         -ctkversion     => CTK_VERSION,
     );
-    
+
     my $status = $h->build();
-    
+
 
 =head1 DESCRIPTION
 
 Helper for building CTK scripts
 
+=head2 new
+
+    my $h = new CTK::Helper (
+        -type           => TYPE,
+        -projectname    => PROJECTNAME,
+        -ctkversion     => CTK_VERSION,
+    );
+
+Returns helper's object
+
+=head2 build
+
+    my $status = $h->build();
+
+Building the project
+
+=head2 backward_build
+
+    my $status = $self->backward_build();
+
+Second pass of building. For internal use only
+
 =head1 SEE ALSO
 
 L<ctklib>, L<ctklib-tiny>
 
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2014 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -99,11 +117,12 @@ use constant {
             PODSIG  => '=',
             DOLLAR  => '$',
             GMT     => sprintf("%s GMT", scalar(gmtime)),
+            YEAR    => (gmtime)[5]+1900,
         },
 };
 
 use vars qw/$VERSION/;
-$VERSION = '2.67';
+$VERSION = '2.68';
 
 sub new {
     my $class = shift;
@@ -113,10 +132,10 @@ sub new {
         ['CTKVERSION','VERSION'],
     ],@_) if defined $_[0];
     my $c = CTKx->instance->c();
-    
+
     my $subclass = SUBCLASSES->{$type};
     croak "Class for $type project could not be loaded, or is an invalid name" unless $subclass;
-    
+
     unless ( $subclass->can('build') ) {
         try {
             eval "require $subclass";
@@ -126,11 +145,13 @@ sub new {
             croak "Class $subclass could not be loaded. $_";
         };
     }
-    
+
     my %rplc = %{(STDRPLC)};
-    $rplc{PROJECTNAME}  = $projectname || PROJECTNAME;
-    $rplc{CTKVERSION}   = $ctkversion || $c->VERSION();
-    
+    $rplc{PROJECTNAME}    = $projectname || PROJECTNAME;
+    $rplc{PROJECTNAMEUCF} = ucfirst($rplc{PROJECTNAME});
+    $rplc{PROJECTNAMEL}   = lc($rplc{PROJECTNAME});
+    $rplc{CTKVERSION}     = $ctkversion || $c->VERSION();
+
     return bless {
             type    => $type,
             projectname => $projectname || PROJECTNAME,
@@ -143,11 +164,10 @@ sub new {
 }
 sub build {
     my $self = shift;
-    my $CRLF = _crlf();
-    
+
     #   
     my @dirs = $self->dirs() if $self->can('dirs');
-    
+
     if (@dirs && ref($dirs[0]) eq 'HASH') {
         $self->{dirs} = [@dirs];
     } elsif (@dirs && ref($dirs[0]) eq 'ARRAY') {
@@ -166,23 +186,24 @@ sub build {
         my $file = ($r =~ /^\s*file\s*\:\s*(.+?)\s*$/mi) ? $1 : '';
         my $mode = ($r =~ /^\s*mode\s*\:\s*(.+?)\s*$/mi) ? $1 : '';
         my $data = ($r =~ /\s*\r?\n\s*\r?\n(.+)/s) ? $1 : '';
+
         $mode = undef unless $mode =~ /^[0-9]{1,3}$/;
         $r = {
                 name => $name,
                 file => $file,
-                data => $data,
+                data => lf_normalize($data),
                 mode => defined $mode ? oct($mode) : undef,
             };
     }
     $self->{pool} = [@pool];
-    
+
     my $ret = 0;
     my $bret = 0;
-    
+
     #    
     $ret = $self->maybe::next::method();
     return 0 unless $ret;
-    
+
     #  
     $bret = $self->backward_build();
     return $bret;
@@ -191,7 +212,7 @@ sub backward_build {
     my $self = shift;
     my $c = CTKx->instance->c();
     my $rplc = $self->{rplc};
-    
+
     #  
     my $dirs = $self->{dirs};
     my $cd = cwd();
@@ -201,7 +222,7 @@ sub backward_build {
         my $mode = defined $d->{mode} ? $d->{mode} : DIRMODE;
         preparedir($path,$mode);
     }
-    
+
     #       
     my $pool = $self->{pool};
     my $overwrite = "yes";
@@ -212,7 +233,7 @@ sub backward_build {
             next;
         }
         my $file = CTK::catfile($cd, split(/\//,_ff($f->{file},$rplc)));
-        $overwrite = $c->cli_prompt("File \"$file\" already exists. Overwrite?:", $overwrite) if -e $file;
+        $overwrite = $c->cli_prompt("File \"$file\" already exists. Overwrite? :", $overwrite) if -e $file;
         if ($overwrite =~ /^y/i) {
             $overwrite = 'yes';
         } else {
@@ -221,7 +242,7 @@ sub backward_build {
         }
         my $mode = $f->{mode};
         my $data = _ff($f->{data},$rplc);
-        fsave($file,$data);
+        bsave($file,$data);
         chmod($mode,$file) if defined($mode);
     }
 
@@ -240,13 +261,5 @@ sub _ff {
     $d =~ s/\%(\w+?)\%/(defined $h->{$1} ? $h->{$1} : '')/eg;
     return $d
 }
-sub _crlf {
-    # Original: CGI::Simple
-    my $OS = $^O || do { require Config; $Config::Config{'osname'} };
-    return
-        ( $OS =~ m/VMS/i )   ? "\n"
-        : ( "\t" ne "\011" ) ? "\r\n"
-        :                      "\015\012";
-}
 1;
 __END__
