@@ -1,11 +1,11 @@
-package CTK::Helper::SkelModule; # $Id: SkelModule.pm 184 2015-05-12 10:15:56Z minus $
+package CTK::Helper::SkelModule; # $Id: SkelModule.pm 200 2017-05-01 08:51:48Z minus $
 use strict;
 
 use CTKx;
 use CTK::Util qw/ :BASE /;
 
 use vars qw($VERSION);
-$VERSION = '1.01';
+$VERSION = '1.02';
 
 sub build {
     #  
@@ -15,27 +15,27 @@ sub build {
     $rplc->{FOO} = "FOO";
     $rplc->{BAR} = "BAR";
     $rplc->{BAZ} = "BAZ";
-    
+
     return 1;
 }
 sub dirs {
     #         
-    
+
     {
         path => '%PROJECTNAME%',
         mode => 0755,
     },
-    
+
     {
         path => '%PROJECTNAME%/bin',
         mode => 0755,
     },
-    
+
     {
         path => '%PROJECTNAME%/inc',
         mode => 0755,
     },
-    
+
     {
         path => '%PROJECTNAME%/lib',
         mode => 0755,
@@ -45,17 +45,47 @@ sub dirs {
         path => '%PROJECTNAME%/lib/CTKx',
         mode => 0755,
     },
-    
+
     {
         path => '%PROJECTNAME%/src',
         mode => 0755,
     },
-    
+
     {
         path => '%PROJECTNAME%/src/conf',
         mode => 0755,
     },
-    
+
+    {
+        path => '%PROJECTNAME%/src/misc',
+        mode => 0755,
+    },
+
+    {
+        path => '%PROJECTNAME%/src/misc/etc',
+        mode => 0755,
+    },
+
+    {
+        path => '%PROJECTNAME%/src/misc/etc/cron.d',
+        mode => 0755,
+    },
+
+    {
+        path => '%PROJECTNAME%/src/misc/etc/default',
+        mode => 0755,
+    },
+
+    {
+        path => '%PROJECTNAME%/src/misc/etc/init.d',
+        mode => 0755,
+    },
+
+    {
+        path => '%PROJECTNAME%/src/misc/etc/logrotate.d',
+        mode => 0755,
+    },
+
     {
         path => '%PROJECTNAME%/t',
         mode => 0755,
@@ -74,7 +104,7 @@ __DATA__
 
 -----BEGIN FILE-----
 Name: %PROJECTNAME%
-File: %PROJECTNAME%/bin/%PROJECTNAME%
+File: %PROJECTNAME%/bin/%PROJECTNAMEL%
 Mode: 777
 
 #!/usr/bin/perl -w
@@ -82,7 +112,7 @@ use strict;
 
 %PODSIG%head1 NAME
 
-%PROJECTNAME% - blah-blah-blah
+%PROJECTNAMEL% - blah-blah-blah
 
 %PODSIG%head1 VERSION
 
@@ -90,56 +120,59 @@ Version 1.00
 
 %PODSIG%head1 SYNOPSIS
 
-    %PROJECTNAME% [options] [commands [args]] 
+    %PROJECTNAMEL% [options] [commands [args]]
 
-    %PROJECTNAME% [-lcdt]
-
-    %PROJECTNAME% [-h | -v | -m]
-
-    %PROJECTNAME% [--help | --version | --man]
-    
-    %PROJECTNAME% [--log] [--logclear] [--debug] [--testmode] [--signature=MESSAGE]
-           [ test | void ]
+    %PROJECTNAMEL% [-ldtv] [--logclear] [--signature=MESSAGE]
+        [ test | void ]
 
 %PODSIG%head1 OPTIONS
 
 %PODSIG%over 8
 
-%PODSIG%item B<-c, --logclear>
+%PODSIG%item B<-c CONFIG_FILE, --config=CONFIG_FILE>
 
-    CTK (ctklib.log)    .   
-       .   , ,
-       .
+Use CONFIG_FILE as configuration file
+
+%PODSIG%item B<--datadir=DIR, --workdir=DIR>
+
+Use DIR as DataDir directory
 
 %PODSIG%item B<-d, --debug>
 
-      CTK.
-         ,    
-   CTK (ctklib.log)    --log.
+Print debug information on STDOUT
 
 %PODSIG%item B<-h, --help>
 
-   .
+Show short help information and quit
+
+%PODSIG%item B<-H, --longhelp>
+
+Show long help information and quit
 
 %PODSIG%item B<-l, --log>
 
-    (debug)      CTK (ctklib.log).
+Writing debug information in the general log-file - %PROJECTNAMEL%.log
 
-%PODSIG%item B<-m, --man>
+%PODSIG%item B<--logclear>
 
-   .
+Cleraing the %PROJECTNAMEL%.log file before the program's run
 
 %PODSIG%item B<--signature=MESSAGE>
 
-       CTK (ctklib.log)  MESSAGE.
+Mark each line of the log file (%PROJECTNAMEL%.log) by a message with MESSAGE.
 
 %PODSIG%item B<-t, --testmode>
 
-    .
+Enable test mode. Not recommended, because the test mode suppress any
+modifications in database and on your hard disk
 
-%PODSIG%item B<-v, --ver, --version>
+%PODSIG%item B<-v, --verbose>
 
-     .
+Verbose option. Include Verbose debug data in the STDOUT and to error-log output
+
+%PODSIG%item B<-V, --version>
+
+Print the version number of the program and quit
 
 %PODSIG%back
 
@@ -149,11 +182,15 @@ Version 1.00
 
 %PODSIG%item B<test>
 
-    .
+Test of the project
+
+    %PROJECTNAMEL% test
 
 %PODSIG%item B<void>
 
- ,      .
+The program runs and does nothing
+
+    %PROJECTNAMEL% void
 
 %PODSIG%back
 
@@ -201,11 +238,11 @@ The usual warnings if it can't read or write the files involved.
 
 %PODSIG%head1 AUTHOR
 
-Your Name (Your Nickname) L<http://your.site> E<lt>your@email.comE<gt>
+Your Name (Your Nickname) L<http://www.your.site> E<lt>your@email.comE<gt>
 
 %PODSIG%head1 COPYRIGHT
 
-Copyright (C) 1998-2015 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-%YEAR% D&D Corporation. All Rights Reserved
 
 %PODSIG%head1 LICENSE
 
@@ -226,10 +263,9 @@ See C<LICENSE> file
 %PODSIG%cut
 
 use constant {
-    PIDFILE   => '%PROJECTNAME%.pid', #  PID  
-    LOGFILE   => '%PROJECTNAME%.log', #  LOG  
+    PIDFILE   => '%PROJECTNAMEL%.pid',
+    LOGFILE   => '%PROJECTNAMEL%.log',
 
-    #    .
     CMDDEFAULT => '', # 'void',
     CMD => {
         void => {
@@ -245,64 +281,93 @@ use constant {
     },
 };
 
-use FindBin qw($RealBin);
-use lib "$RealBin/inc";
 use Getopt::Long;
 use Pod::Usage;
 
 # CTK Packages
-use base qw/CTKx::%PROJECTNAME%/;
+use base qw/CTKx::%PROJECTNAMEUCF%/;
 use CTK;
 use CTKx;
 use CTK::FilePid;
 
-#  
 Getopt::Long::Configure ("bundling");
-
 GetOptions(\%OPT,
 
-    #  
-    @OPTSYSDEF, # humvdlcyt?
+    #
+    # NoUsed keys map:
+    #
+    # a A b B   C   D e E
+    # f F g G     i I j J
+    # k K   L   M n N o O
+    # p P q Q r R s S   T
+    # u U     w W x X y Y
+    # z Z
+    #
 
-    #   
-    "foo|f=s",            # FOO
-    "bar|b=i",            # BAR
-    "baz|z=s",            # BAZ
-    
-) || pod2usage(-exitval => 1, -verbose => 0, -output => \*CTK::CTKCP);
-pod2usage(-exitval => 0, -verbose => 0, -output => \*CTK::CTKCP) if $OPT{help};
-pod2usage(-exitval => 0, -verbose => 99, -sections => 'NAME|VERSION', -output => \*CTK::CTKCP) if $OPT{version};
-pod2usage(-exitval => 0, -verbose => 2, -output => \*CTK::CTKCP) if $OPT{man};
+    # Help
+    "help|usage|h|?",       # Show short help page
+    "longhelp|H",           # Show long help page
+    "version|ver|V",        # Show VERSION
+
+    # Debugging
+    "debug|d",              # Debug mode
+    "log|l",                # Log mode
+    "verbose|v",            # Verbose mode. Default: Silent mode
+    "logclear|logclean",    # Clear log
+    "signature=s",          # Sign to log
+    "testmode|test|t",      # Testmode
+
+    # CTK
+    "cfgfile|config|c=s",   # CfgFile
+    "datadir|workdir=s",    # DataDir
+
+    # Application
+    "foo|f=s",              # FOO
+    "bar|b=i",              # BAR
+    "baz|z=s",              # BAZ
+
+) || pod2usage(-exitval => 1, -verbose => 0, -output => \*STDERR);
+pod2usage(-exitval => 0, -verbose => 1) if $OPT{help};
+pod2usage(-exitval => 0, -verbose => 2) if $OPT{longhelp};
+say(CTKx::%PROJECTNAMEUCF%->VERSION) && exit(0) if $OPT{version};
 
 # VARS
 my %cmddata;
 
-# 
-my $command   = @ARGV ? shift @ARGV : CMDDEFAULT; # 
-my @arguments = @ARGV ? @ARGV : (); #  
-my @commands  = keys %{sub{CMD}->()}; # @{sub{COMMANDS}->()}
-pod2usage(-exitval => 1, -verbose => 99, -sections => 'SYNOPSIS|OPTIONS|COMMANDS', -output => \*CTK::CTKCP)
+# Commands
+my $command   = @ARGV ? shift @ARGV : CMDDEFAULT;
+my @arguments = @ARGV ? @ARGV : ();
+my @commands  = keys %{sub{CMD}->()};
+pod2usage(-exitval => 1, -verbose => 99, -sections => 'SYNOPSIS|OPTIONS|COMMANDS', -output => \*STDERR)
     if ( (grep {$_ eq $command} @commands) ? 0 : 1 );
-    
-#       Singleton
-my $c = new CTK( syspaths => 1 );
+
+# Singleton object
+my $c = defined($OPT{cfgfile})
+        ? new CTK(
+                cfgfile => $OPT{cfgfile},
+                logdir  => '.',
+                datadir  => '.'
+            )
+        : new CTK( syspaths => 1 );
+$c->datadir($OPT{datadir}) if defined($OPT{datadir});
 my $ctkx = CTKx->instance( c => $c );
 
-# unlink( $LOGFILE ) if( $OPT{logclear} && -e $LOGFILE ); # Remove unnesessary log
+unlink( $LOGFILE ) if( $OPT{logclear} && -e $LOGFILE ); # Remove unnesessary log
 
 START:  debug "-"x16, " START ", (testmode() ? 'IN TEST MODE ' : ''), tms," ","-"x16;
 #########################
 ### START
 #########################
+my $pidfile;
 my $code = __PACKAGE__->can(uc($command)) || __PACKAGE__->can($command);
 if ($code && ref($code) eq 'CODE') {
     %cmddata = %{CMD->{$command}};
     $cmddata{arguments} = [@arguments];
 
     CTK::preparedir({ datadir => $c->datadir() });
-    
-    #  LOG    
-    $c->logfile(CTK::catfile($c->logdir(), 
+
+    # Log File define
+    $c->logfile(CTK::catfile($c->logdir(),
             CTK::dformat($cmddata{logfile}, {
                         COMMAND  => $command,
                         PREFIX   => $c->prefix(),
@@ -314,23 +379,23 @@ if ($code && ref($code) eq 'CODE') {
             )
         ) if $cmddata{logfile};
 
-    #  PID    
-    my $pidfile = new CTK::FilePid({ file => CTK::catfile($c->datadir(), $cmddata{pidfile} || PIDFILE) });
+    # PID File define
+    $pidfile = new CTK::FilePid({ file => CTK::catfile($c->datadir(), $cmddata{pidfile} || PIDFILE) });
     my $pidstat = $pidfile->running || 0;
-    
+
     debug "==== START COMMAND: ".uc($command)." ($$) ====";
-    
+
     if ($cmddata{pidcheck}) {
         exception("PID STATE (".$pidfile->file()."): ALREADY EXISTS (PID: $pidstat)" ) if $pidstat;
         $pidfile->write;
     }
 
-    &{$code}(%cmddata); #       
-    
+    &{$code}(%cmddata);
+
     if ($cmddata{pidcheck}) {
         $pidfile->remove;
     }
-    
+
     debug "==== FINISH COMMAND: ".uc($command)." ($$) ====";
 } else {
     exception("Sub \"".uc($command)."\" undefined");
@@ -344,15 +409,6 @@ exit(0);
 
 1;
 __END__
-
-NoUsed keys map:
-
-a A b B   C   D e E 
-f F g G   H i I j J 
-k K   L   M n N o O 
-p P q Q r R s S   T 
-  U   V w W x X   Y 
-z Z
 -----END FILE-----
 
 -----BEGIN FILE-----
@@ -360,16 +416,16 @@ Name: FakeCTK.pm
 File: %PROJECTNAME%/inc/FakeCTK.pm
 Mode: 644
 
-package FakeCTK; # $Id: SkelModule.pm 184 2015-05-12 10:15:56Z minus $
+package FakeCTK; # $Id: SkelModule.pm 200 2017-05-01 08:51:48Z minus $
 use strict;
 
 %PODSIG%head1 NAME
 
-FakeCTK -   CTK     CTKlib
+FakeCTK - Module simulating CTK for testing CTKlib projects
 
 %PODSIG%head1 VERSION
 
-Version 1.00
+Version 1.01
 
 %PODSIG%head1 SYNOPSIS
 
@@ -377,15 +433,15 @@ Version 1.00
 
 %PODSIG%head1 DESCRIPTION
 
-  CTK     CTKlib    
+Module simulating CTK for testing CTKlib projects in your test scripts
 
 %PODSIG%head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) http://www.serzik.ru <minus@mail333.com>
+Sergey Lepenkov (Serz Minus) http://www.serzik.com <minus@mail333.com>
 
 %PODSIG%head1 COPYRIGHT
 
-Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 %PODSIG%head1 LICENSE
 
@@ -395,12 +451,21 @@ under the same terms as Perl itself.
 %PODSIG%cut
 
 use vars qw/$VERSION/;
-$VERSION = '1.00';
+$VERSION = '1.01';
+
+use constant {
+    PROJECTNAME => '%PROJECTNAME%',
+};
 
 use CTK;
 use CTKx;
 
-CTKx->instance( c => new CTK( syspaths => 1 ) );
+my $c = new CTK(
+    syspaths    => 1,
+    prefix      => lc(PROJECTNAME),
+);
+
+CTKx->instance( c => $c );
 
 1;
 __END__
@@ -411,7 +476,7 @@ Name: PostConf.pm
 File: %PROJECTNAME%/inc/PostConf.pm
 Mode: 644
 
-package PostConf; # $Id: SkelModule.pm 184 2015-05-12 10:15:56Z minus $
+package PostConf; # $Id: SkelModule.pm 200 2017-05-01 08:51:48Z minus $
 use strict;
 
 %PODSIG%head1 NAME
@@ -420,25 +485,27 @@ PostConf - Configuration your modules on phase Postamble.
 
 %PODSIG%head1 VERSION
 
-Version 1.00
+Version 1.01
 
 %PODSIG%head1 SYNOPSIS
 
-    perl -MPostConf -e configure -- PROJECTNAME
+    perl -Iinc -MPostConf -e configure -- PROJECTNAME
+
+    perl -Iinc -MPostConf -e install -- PROJECTNAME
 
 %PODSIG%head1 DESCRIPTION
 
 B<FOR INTERNAL USE ONLY!>
 
-       postamble
+Configuration your modules on phase Postamble.
 
 %PODSIG%head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) http://www.serzik.ru <minus@mail333.com>
+Sergey Lepenkov (Serz Minus) http://www.serzik.com <minus@mail333.com>
 
 %PODSIG%head1 COPYRIGHT
 
-Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 %PODSIG%head1 LICENSE
 
@@ -448,43 +515,229 @@ under the same terms as Perl itself.
 %PODSIG%cut
 
 use base qw/Exporter/;
-our @EXPORT = qw/configure/;
+our @EXPORT = qw/configure install/;
 
 use vars qw/$VERSION/;
-$VERSION = '1.00';
+$VERSION = '1.01';
 
-use CTK qw/ :NONE /;
+use CTK qw/ say /;
 use CTK::Util qw/ :BASE /;
-use File::Copy qw/copy/;
+use File::Copy qw/copy cp/;
+use File::Find;
+use Cwd;
+my $basedir = getcwd();
 
 sub configure {
     my $projectname = @ARGV ? shift @ARGV : '';
     croak "Project name is missing!" unless $projectname;
-    
-    my $c = new CTK( prefix=> $projectname, syspaths => 1 );
+
+    my $c = new CTK( prefix=> lc($projectname), syspaths => 1 );
     my $confdir = $c->confdir();
     my $cfgfile = $c->cfgfile();
-    if ($cfgfile && -e $cfgfile) {
-        my $overwrite = $c->cli_prompt('Configuration files already exists. Do you want to overwrite it?:', 'No');
-        return 0 unless $overwrite && $overwrite =~ /^\s*y/i;
+    my $datadir = $c->datadir();
+
+    preparedir({
+       confdir => $confdir,
+       datadir => $datadir,
+    });
+
+    if ($cfgfile && !-e $cfgfile) {
+        printf "Copying file %s --> %s... ", catfile('blib','conf',lc($projectname).'.conf'), $cfgfile;
+        copy(catfile('blib','conf',lc($projectname).'.conf'), $cfgfile);
+        print "Done.\n";
     }
+    copy($cfgfile, $cfgfile.".default") if !-e $cfgfile.".default";
+
+    # blib/conf -> /etc/PROJECTNAME/conf
+    my $wdir = catdir('blib','conf','conf');
+    chdir($wdir) or do {warn "Can't change directory: $!"; return};
+    find({ wanted => sub {
+        return if /^\.exists$/;
+        if (-f $_) {
+            my $src = catfile($wdir,$File::Find::dir, $_);
+            my $dst = catfile($confdir, $File::Find::dir, $_);
+            printf "Copying file %s --> %s... ", $src, $dst;
+            if (-e $dst) {
+                print "Skipped. File already exists\n";
+                return;
+            }
+            cp($_, $dst) or warn "Can't create $dst: $!";
+            print "Done.\n";
+        } elsif (-d $_) {
+            return if /^\.+$/;
+            my $dst = catdir($confdir, $File::Find::dir, $_);
+            my $perm = 0755 & 07777;
+            print sprintf("Creating directory %s [%03o]... ", $dst, $perm);
+            if (-e $dst) {
+                print "Skipped. Directory already exists\n";
+                return;
+            }
+            mkdir $dst or warn "Can't create $dst: $!";;
+            eval { chmod $perm, $dst; };
+            print "Done.\n";
+        } else {
+            say "Skipped: $_";
+        }
+    }}, ".");
+    chdir($basedir) or warn "Can't change directory: $!";
 
-    preparedir($confdir);
-    copy(catfile('src',$projectname.'.conf'), catfile($cfgfile));
-    copy( $cfgfile, $cfgfile.".default");
-    $c->fcopy( -in => catdir('src','conf'), -out => $confdir, -file => qr/\.conf(\.sample$|$)/ );
     return 1;
 }
+sub install {
+    my $projectname = @ARGV ? shift @ARGV : '';
+    croak "Project name is missing!" unless $projectname;
+
+    # blib/misc
+    # Misc (for *nix only)
+    if (isostype('Unix')) {
+        say sprintf("Embedding %s...", $projectname);
+        my $rootdir = rootdir();
+        my $wdir = catdir('blib','misc');
+        chdir($wdir) or do {warn "Can't change directory: $!"; return};
+        find({ wanted => sub {
+            return if /^\.exists$/;
+            if (-f $_) {
+                my $dir = catdir($rootdir, $File::Find::dir);
+                my $src = catfile($wdir,$File::Find::dir, $_);
+                my $dst = catfile($dir, $_);
+                printf "Copying file %s --> %s... ", $src, $dst;
+                unless (-e $dir) {
+                    print "Skipped. Directory $dir not found\n";
+                    return;
+                }
+                if (-e $dst) {
+                    print "Skipped. File already exists\n";
+                    return;
+                }
+                cp($_, $dst) or warn "Can't copy $dst: $!";
+                print "Done.\n";
+            } elsif (-d $_) {
+                return if /^\.+$/;
+                my $dst = catdir($rootdir, $File::Find::dir, $_);
+                my $perm = 0755 & 07777;
+                print sprintf("Creating directory %s [%03o]... ", $dst, $perm);
+                if (-e $dst) {
+                    print "Skipped. Directory already exists\n";
+                    return;
+                }
+                mkdir $dst or warn "Can't create $dst: $!";
+                eval { chmod $perm, $dst; };
+                print "Done.\n";
+            } else {
+                say "Skipped: $_";
+            }
+        }}, ".");
+        chdir($basedir) or warn "Can't change directory: $!";
+    }
+
+    return 1;
+}
+
 1;
 __END__
 -----END FILE-----
 
 -----BEGIN FILE-----
-Name: %PROJECTNAME%.pm
-File: %PROJECTNAME%/lib/CTKx/%PROJECTNAME%.pm
+Name: PreConf.pm
+File: %PROJECTNAME%/inc/PreConf.pm
 Mode: 644
 
-package CTKx::%PROJECTNAME%; # %DOLLAR%Id%DOLLAR%
+package PreConf; # $Id: SkelModule.pm 200 2017-05-01 08:51:48Z minus $
+use strict;
+
+%PODSIG%head1 NAME
+
+PreConf - Configuration your modules on phase Preamble.
+
+%PODSIG%head1 VERSION
+
+Version 1.01
+
+%PODSIG%head1 SYNOPSIS
+
+    perl -Iinc -MPreConf -e dircp -- SRCDIR DSTDIR
+
+    perl -Iinc -MPreConf -e crlfnorm -- DIR_OR_FILE_NAMES
+
+%PODSIG%head1 DESCRIPTION
+
+PreConf is a perl module that runs pre configuration tasks on phase Preamble.
+
+B<FOR INTERNAL USE ONLY!>
+
+%PODSIG%head1 AUTHOR
+
+Sergey Lepenkov (Serz Minus) http://www.serzik.com <minus@serzik.com>
+
+%PODSIG%head1 COPYRIGHT
+
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
+
+%PODSIG%head1 LICENSE
+
+This is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
+
+%PODSIG%cut
+
+use base qw/Exporter/;
+our @EXPORT = qw/dircp crlfnorm/;
+
+use vars qw/$VERSION/;
+$VERSION = '1.01';
+
+use CTK qw/ say /;
+use CTK::Util qw/ :BASE /;
+use File::Copy qw/copy cp/;
+use File::Copy::Recursive qw/dircopy/;
+use File::Find;
+
+sub dircp {
+    _expand_wildcards();
+    my @src = @ARGV;
+    my $dst = pop @src;
+    croak("Too many arguments") if (@src > 1 && ! -d $dst);
+
+    my $nok = 0;
+    foreach my $src (@src) {
+        if (-f $src) {
+            $nok ||= !cp($src, $dst);
+        } elsif (-d $src) {
+            $nok ||= !dircopy($src, $dst);
+        } else { # Skipped
+            say "Skip $src. This is not file and not directory";
+        }
+    }
+    return $nok;
+}
+sub crlfnorm {
+    find({ wanted => sub {
+        return if -d;
+        return unless -w _;
+        return unless -r _;
+        return if -B _;
+        printf("Normalizing the linefeeds in file %s... ", catfile($File::Find::dir, $_));
+        file_lf_normalize($_);
+        print "Done.\n";
+    }}, @ARGV);
+    return 1;
+}
+
+sub _expand_wildcards {
+    # Original in package ExtUtils::Command
+    @ARGV = map(/[*?]/o ? glob($_) : $_, @ARGV);
+}
+
+1;
+__END__
+-----END FILE-----
+
+-----BEGIN FILE-----
+Name: %PROJECTNAMEUCF%.pm
+File: %PROJECTNAME%/lib/CTKx/%PROJECTNAMEUCF%.pm
+Mode: 644
+
+package CTKx::%PROJECTNAMEUCF%; # %DOLLAR%Id%DOLLAR%
 use strict;
 
 use vars qw/$VERSION/;
@@ -495,35 +748,35 @@ use CTK qw/:BASE/;
 use Data::Dumper; $Data::Dumper::Deparse = 1;
 
 sub void {
-    #  
     debug("VOID CONTEXT");
     1;
 }
 sub test {
-    #  
-    debug(" ");
+    debug("Testing...");
     debug;
-    
-    #    CTK   
-    my $c = CTKx->instance->c; 
+
+    my $c = CTKx->instance->c;
     my %cmd = @_;
-    
+
     my @arguments = @{$cmd{arguments}};
-    debug(" : ", join("; ",@arguments) || 'none' );
+
+    debug("Argumens: ", join("; ",@arguments) || 'none' );
     $c->info("Arguments:", @arguments);
 
-    #       CTK
-    my $config = $c->config;
-    debug(" CTK:");
-    debug(Dumper($c));
-
-    debug(" :");
+    debug("Input params:");
     debug(Dumper(\%cmd));
-    
+
+    # CTK debug
+    my $config = $c->config;
+    if (verbosemode) {
+        debug("CTK object:");
+        debug(Dumper($c));
+    }
+
     #
-    # ...
+    # . . .
     #
-    
+
     1;
 }
 
@@ -532,8 +785,8 @@ __END__
 -----END FILE-----
 
 -----BEGIN FILE-----
-Name: %PROJECTNAME%.conf
-File: %PROJECTNAME%/src/%PROJECTNAME%.conf
+Name: %PROJECTNAMEL%.conf
+File: %PROJECTNAME%/src/%PROJECTNAMEL%.conf
 Mode: 666
 
 ################################################################################
@@ -554,47 +807,44 @@ Mode: 666
 
 ################################################################################
 #
-#     sendmail     
+# SendMail section
 #
 ################################################################################
 <SendMail>
-    #    
     to          to@example.com
     cc          cc@example.com
     from        from@example.com
-    
-    #      
+
     testmail    test@example.com
     errormail   error@example.com
-    
-    #         UTF-8
+
+    # Mail source encoding for to UTF-8 converting
     charset     windows-1251
     type        text/plain
-    
-    #  sendmail   
+
+    # sendmail and arguments
     sendmail    /usr/sbin/sendmail
     flags       -t
-    
-    # SMTP ,  . 
-    # SMTP      sendmail
+
+    # SMTP server address
     smtp        192.168.1.1
-    
-    #  SMTP
+
+    # Authorization SMTP data (optional)
     #smtpuser user
     #smtppass password
 
-    #   (    )
+    # Attach files
     #<Attach>
-    #    Filename    doc1.txt 
-    #    Type        text/plain 
+    #    Filename    doc1.txt
+    #    Type        text/plain
     #    Disposition attachment
     #    Data        "Document 1. Content"
     #</Attach>
     #<Attach>
-    #    Filename    mbutiny.log
+    #    Filename    %PROJECTNAMEL%.log
     #    Type        text/plain
     #    Disposition attachment
-    #    Path        /var/log/mbutiny.log
+    #    Path        /var/log/%PROJECTNAMEL%.log
     #</Attach>
 </SendMail>
 -----END FILE-----
@@ -606,7 +856,7 @@ Mode: 666
 
 ################################################################################
 #
-#     Oracle/MySQL/SQLite/PGSQL
+# DBI database section: Oracle/MySQL/SQLite/PGSQL and etc.
 #
 ################################################################################
 
@@ -630,40 +880,7 @@ Mode: 666
 
 ################################################################################
 #
-#    .  .   CTK
-# 
-#         ,
-#        
-# .       :
-#
-#      :
-#    FILE     --     
-#    FILENAME --    
-#    DIRSRC   --    
-#    DIRIN    -- = DIRSRC
-#    DIRDST   --     
-#    DIROUT   -- = DIRDST
-#    LIST     --    ,  
-#    EXC      -- 'exclude file' !!!!!!
-#
-#        :
-#    FILE     --       
-#    DIRSRC   --        
-#    DIRIN    -- = DIRSRC
-#    LIST     --    ,  
-#    EXC      -- 'exclude file' !!!!!!
-#
-#        tar
-# 
-# <Arc tgz> #   . ,  ,    
-#    type       tar                       #  ,   
-#    ext        tgz                       #   
-#    create     tar -zcpf [FILE] [LIST]   #    
-#    extract    tar -zxpf [FILE] [DIRDST] #      
-#    exclude    --exclude-from            # !!!!!!
-#    list       tar -ztf [FILE]           #       
-#    nocompress tar -cpf [FILE]           #      
-# </Arc>
+# Archivation section
 #
 ################################################################################
 
@@ -715,9 +932,9 @@ Mode: 666
     nocompress zip -qq0
 </Arc>
 
-# bzip2 
+# bzip2
 <Arc bz2>
-    type       bzip2 
+    type       bzip2
     ext        bz2
     create     bzip2 --best [FILE] [LIST]
     extract    bzip2 -d [FILE]
@@ -748,10 +965,10 @@ Mode: 644
 #!/usr/bin/perl -w
 #########################################################################
 #
-# Serz Minus (Lepenkov Sergey), <minus@mail333.com>
+# Sergey Lepenkov (Serz Minus), <minus@mail333.com>
+#
+# Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 #
-# Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
-# 
 # This is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 #
@@ -760,9 +977,9 @@ Mode: 644
 #########################################################################
 use Test::More tests => 2;
 use lib qw(inc);
-BEGIN { use_ok('CTKx::%PROJECTNAME%') };
+BEGIN { use_ok('CTKx::%PROJECTNAMEUCF%') };
 use FakeCTK;
-ok(CTKx::%PROJECTNAME%::void(),'CTKx::%PROJECTNAME%::void()');
+ok(CTKx::%PROJECTNAMEUCF%::void(),'CTKx::%PROJECTNAMEUCF%::void()');
 1;
 -----END FILE-----
 
@@ -778,16 +995,7 @@ VERSION
     Version 1.00
 
 SYNOPSIS
-    %PROJECTNAME% [options] [commands [args]] 
-
-    %PROJECTNAME% [-lcdt]
-
-    %PROJECTNAME% [-h | -v | -m]
-
-    %PROJECTNAME% [--help | --version | --man]
-    
-    %PROJECTNAME% [--log] [--logclear] [--debug] [--testmode] [--signature=MESSAGE]
-           [ test | void ]
+    %PROJECTNAME% [options] [commands [args]]
 
 ABSTRACT
     The project ...
@@ -817,10 +1025,10 @@ THANKS
     Thanks to me for ...
 
 AUTHOR
-    Your Name (Your Nickname) http://your.site <your@email.com>
+    Your Name (Your Nickname) http://www.your.site <your@email.com>
 
 COPYRIGHTS
-    Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
+    Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 LICENSE
     This program is distributed under the GNU GPL v3.
@@ -924,31 +1132,44 @@ my $build_requires = {
     };
 
 my $prereq_pm = {
-        'CTK'           => %CTKVERSION%,
+        'CTK'                   => %CTKVERSION%,
     };
 
 my $exefiles = [qw(
-        bin/%PROJECTNAME%
+        bin/%PROJECTNAMEL%
     )];
 
 WriteMakefile(
     'NAME'              => '%PROJECTNAME%',
     'DISTNAME'          => '%PROJECTNAME%',
-    'MIN_PERL_VERSION'  => 5.008001,
-    'VERSION_FROM'      => 'lib/CTKx/%PROJECTNAME%.pm',
-    'ABSTRACT_FROM'     => 'bin/%PROJECTNAME%',
+    'MIN_PERL_VERSION'  => 5.010001,
+    'VERSION_FROM'      => 'lib/CTKx/%PROJECTNAMEUCF%.pm',
+    'ABSTRACT_FROM'     => 'bin/%PROJECTNAMEL%',
     'BUILD_REQUIRES'    => $build_requires,
     'PREREQ_PM'         => $prereq_pm,
     'EXE_FILES'         => $exefiles,
-    'AUTHOR'            => 'Your Name (Your Nickname) http://your.site <your@email.com>',
+    'AUTHOR'            => 'Your Name (Your Nickname) http://www.your.site <your@email.com>',
     'LICENSE'           => 'gpl',
     'META_MERGE'        => {
         resources => {
-            homepage        => 'http://your.site',
-            repository      => 'http://svn.your.site/%PROJECTNAME%',
+            homepage        => 'http://www.your.site',
+            repository      => 'http://svn.your.site/%PROJECTNAME%/trunk',
             license         => 'http://opensource.org/licenses/gpl-license.php',
         },
     },
+    macro => {
+            # Main constants
+            PROJECTNAME => '%PROJECTNAME%',
+            PROJECTNAMEL => '%PROJECTNAMEL%',
+            PROJECTNAMEUCF => '%PROJECTNAMEUCF%',
+
+            # Dirs
+            INST_CONF => 'blib$(DFSEP)conf',
+            INST_MISC => 'blib$(DFSEP)misc',
+        },
+    clean => {
+            FILES => '$(INST_CONF) $(INST_MISC)',
+        },
 );
 1;
 
@@ -956,23 +1177,51 @@ package MY;
 
 sub postamble {
 return <<'MAKE_FRAG';
-.PHONY: configure
 
-PROJECTNAME = %PROJECTNAME%
+DIRCP = $(ABSPERLRUNINST) -Iinc -MPreConf -e dircp --
+CRLFNORM = $(ABSPERLRUNINST) -Iinc -MPreConf -e crlfnorm --
+PCONFIGURE = $(ABSPERLRUNINST) -Iinc -MPostConf -e configure --
+PINSTALL = $(ABSPERLRUNINST) -Iinc -MPostConf -e install --
+
+pure_all :: $(INST_CONF)$(DFSEP).exists $(INST_MISC)$(DFSEP).exists \
+  $(INST_CONF)$(DFSEP)$(PROJECTNAMEL).conf
+	$(NOECHO) $(NOOP)
+
+$(INST_CONF)$(DFSEP).exists :: Makefile.PL
+	$(NOECHO) $(MKPATH) $(INST_CONF)
+	$(NOECHO) $(CHMOD) $(PERM_DIR) $(INST_CONF)
+	$(NOECHO) $(TOUCH) $(INST_CONF)$(DFSEP).exists
+
+$(INST_MISC)$(DFSEP).exists :: Makefile.PL
+	$(NOECHO) $(MKPATH) $(INST_MISC)
+	$(NOECHO) $(CHMOD) $(PERM_DIR) $(INST_MISC)
+	$(NOECHO) $(TOUCH) $(INST_MISC)$(DFSEP).exists
+
+$(INST_CONF)$(DFSEP)$(PROJECTNAMEL).conf : src$(DFSEP)$(PROJECTNAMEL).conf
+	$(CP) src$(DFSEP)$(PROJECTNAMEL).conf $(INST_CONF)$(DFSEP)$(PROJECTNAMEL).conf
+	$(CRLFNORM) $(INST_CONF)$(DFSEP)$(PROJECTNAMEL).conf
+	$(DIRCP) src$(DFSEP)conf $(INST_CONF)$(DFSEP)conf
+	$(DIRCP) src$(DFSEP)misc $(INST_MISC)
+	$(CRLFNORM) $(INST_CONF) $(INST_MISC)
 
 configure : src$(DFSEP).configured
 	$(NOECHO) $(NOOP)
 
-pure_install :: src$(DFSEP).configured
+pure_install :: src$(DFSEP).configured src$(DFSEP).installed
 	$(NOECHO) $(NOOP)
 
-src$(DFSEP).configured : src$(DFSEP)$(PROJECTNAME).conf
-	$(ABSPERLRUNINST) -Iinc -MPostConf -e configure -- $(PROJECTNAME)
+src$(DFSEP).configured : src$(DFSEP)$(PROJECTNAMEL).conf
+	$(PCONFIGURE) $(PROJECTNAME)
 	$(NOECHO) $(TOUCH) src$(DFSEP).configured
 	$(NOECHO) $(ECHO) The project $(PROJECTNAME) was successfully configured.
 
+src$(DFSEP).installed :: Makefile.PL
+	$(PINSTALL) $(PROJECTNAME)
+	$(NOECHO) $(TOUCH) src$(DFSEP).installed
+	$(NOECHO) $(ECHO) The project $(PROJECTNAME) was successfully installed.
+
 clean :: Makefile.PL
-	-$(RM_F) src$(DFSEP).configured
+	-$(RM_F) src$(DFSEP).configured src$(DFSEP).installed
 
 MAKE_FRAG
 }
@@ -987,15 +1236,18 @@ Mode: 644
 # Generated by CTKlib %CTKVERSION%
 # %DOLLAR%Id%DOLLAR%
 
-bin/%PROJECTNAME%
-src/%PROJECTNAME%.conf
+bin/%PROJECTNAMEL%
+src/%PROJECTNAMEL%.conf
 src/conf/extra-arc.conf         Archivator settings
 src/conf/extra-store.conf       Store settings
 src/conf/extra-sendmail.conf    Sendmail settings
+src/misc/etc/default/%PROJECTNAMEL%
+src/misc/etc/logrotate.d/%PROJECTNAMEL%
 inc/FakeCTK.pm                  For testing
-inc/PostConf.pm                 Post configuration module
+inc/PostConf.pm                 Post configuration module (postamble)
+inc/PreConf.pm                  Pre configuration module (preamble)
 t/01-use.t                      Test script
-lib/CTKx/%PROJECTNAME%.pm
+lib/CTKx/%PROJECTNAMEUCF%.pm
 CHANGES                         Changes list
 INSTALL                         Installation instructions
 LICENSE                         License file
@@ -1005,6 +1257,56 @@ README                          !!! README FIRST !!!
 TODO                            TO DO
 -----END FILE-----
 
+-----BEGIN FILE-----
+Name: %PROJECTNAMEL%
+File: %PROJECTNAME%/src/misc/etc/default/%PROJECTNAMEL%
+Mode: 644
+
+#
+# This file must only contain KEY=VALUE lines. Do not use advanced
+# shell script constructs!
+#
+
+## ScriptName (client program name)
+SCRIPTNAME=%PROJECTNAMEL%
+
+-----END FILE-----
+
+-----BEGIN FILE-----
+Name: %PROJECTNAMEL%
+File: %PROJECTNAME%/src/misc/etc/logrotate.d/%PROJECTNAMEL%
+Mode: 644
+
+/var/log/%PROJECTNAMEL%.*.log {
+    daily
+    missingok
+    rotate 14
+    compress
+    delaycompress
+    notifempty
+    sharedscripts
+}
+/var/log/%PROJECTNAMEL%.log {
+    daily
+    missingok
+    rotate 14
+    compress
+    delaycompress
+    notifempty
+    sharedscripts
+}
+/var/log/%PROJECTNAMEL%_*.log {
+    daily
+    missingok
+    rotate 14
+    compress
+    delaycompress
+    notifempty
+    sharedscripts
+}
+
+-----END FILE-----
+
 -----BEGIN FILE-----
 Name: LICENSE
 File: %PROJECTNAME%/LICENSE
