@@ -1,57 +1,48 @@
-package CTK::Helper::SkelRegular; # $Id: SkelRegular.pm 163 2013-10-17 10:13:34Z minus $
+package CTK::Helper::SkelRegular; # $Id: SkelRegular.pm 200 2017-05-01 08:51:48Z minus $
 use strict;
 
 use CTKx;
 use CTK::Util qw/ :BASE /;
 
 use vars qw($VERSION);
-$VERSION = '1.00';
+$VERSION = '1.01';
 
 sub build {
-    #  
     my $self = shift;
     return 1;
 }
 sub dirs {
-    #         
-    
     {
         path => '%PROJECTNAME%',
         mode => 0755,
     },
 
-    {
-        path => '%PROJECTNAME%/bin',
-        mode => 0755,
-    },
-    
     {
         path => '%PROJECTNAME%/inc',
         mode => 0755,
     },
-    
+
     {
         path => '%PROJECTNAME%/src',
         mode => 0755,
     },
-    
+
     {
         path => '%PROJECTNAME%/conf',
         mode => 0755,
     },
-    
+
     {
         path => '%PROJECTNAME%/log',
         mode => 0777,
     },
-    
+
     {
         path => '%PROJECTNAME%/data',
         mode => 0777,
     },
 }
 sub pool {
-    #    multipart 
     my $pos =  tell DATA;
     my $data = scalar(do { local $/; <DATA> });
     seek DATA, $pos, 0;
@@ -62,16 +53,16 @@ sub pool {
 __DATA__
 
 -----BEGIN FILE-----
-Name: %PROJECTNAME%.pl
-File: %PROJECTNAME%/%PROJECTNAME%.pl
+Name: %PROJECTNAMEL%
+File: %PROJECTNAME%/%PROJECTNAMEL%
 Mode: 777
 
 #!/usr/bin/perl -w
-use strict;
+use strict; # %DOLLAR%Id%DOLLAR%
 
 %PODSIG%head1 NAME
 
-%PROJECTNAME%.pl - blah-blah-blah
+%PROJECTNAMEL% - blah-blah-blah
 
 %PODSIG%head1 VERSION
 
@@ -79,53 +70,50 @@ Version 1.00
 
 %PODSIG%head1 SYNOPSIS
 
-    %PROJECTNAME%.pl [options] [commands [args]] 
+    %PROJECTNAMEL% [options] [commands [args]]
 
-    %PROJECTNAME%.pl [-lcdt]
-
-    %PROJECTNAME%.pl [-h | -v | -m]
-
-    %PROJECTNAME%.pl [--help | --version | --man]
-    
-    %PROJECTNAME%.pl [--log] [--logclear] [--debug] [--testmode] [--signature=MESSAGE]
-           [ test | void ]
+    %PROJECTNAMEL% [-ldtv] [--logclear] [--signature=MESSAGE]
 
 %PODSIG%head1 OPTIONS
 
 %PODSIG%over 4
 
-%PODSIG%item B<-c, --logclear>
-
-    CTK    .
-
 %PODSIG%item B<-d, --debug>
 
-      CTK.
-         .
+Print debug information on STDOUT
 
 %PODSIG%item B<-h, --help>
 
-   .
+Show short help information and quit
+
+%PODSIG%item B<-H, --longhelp>
+
+Show long help information and quit
 
 %PODSIG%item B<-l, --log>
 
-    (debug)     CTK.
+Writing debug information in the general log-file - %PROJECTNAMEL%.log
 
-%PODSIG%item B<-m, --man>
+%PODSIG%item B<--logclear>
 
-   .
+Cleraing the %PROJECTNAMEL%.log file before the program's run
 
 %PODSIG%item B<--signature=MESSAGE>
 
-      CTK  MESSAGE.
+Mark each line of the log file (%PROJECTNAMEL%.log) by a message with MESSAGE.
 
 %PODSIG%item B<-t, --testmode>
 
-    .
+Enable test mode. Not recommended, because the test mode suppress any
+modifications in database and on your hard disk
 
-%PODSIG%item B<-v, --ver, --version>
+%PODSIG%item B<-v, --verbose>
 
-     .
+Verbose option. Include Verbose debug data in the STDOUT and to error-log output
+
+%PODSIG%item B<-V, --version>
+
+Print the version number of the program and quit
 
 %PODSIG%back
 
@@ -135,11 +123,15 @@ Version 1.00
 
 %PODSIG%item B<test>
 
-    .
+Test of the project
+
+    %PROJECTNAMEL% test
 
 %PODSIG%item B<void>
 
- ,      .
+The program runs and does nothing
+
+    %PROJECTNAMEL% void
 
 %PODSIG%back
 
@@ -187,11 +179,11 @@ The usual warnings if it can't read or write the files involved.
 
 %PODSIG%head1 AUTHOR
 
-Your Name (Your Nickname) L<http://your.site> E<lt>your@email.comE<gt>
+Your Name (Your Nickname) L<http://www.your.site> E<lt>your@email.comE<gt>
 
 %PODSIG%head1 COPYRIGHT
 
-Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 %PODSIG%head1 LICENSE
 
@@ -212,10 +204,9 @@ See C<LICENSE> file
 %PODSIG%cut
 
 use constant {
-    PIDFILE   => '%PROJECTNAME%.pid', #  PID  
+    PIDFILE   => '%PROJECTNAMEL%.pid',
 
-    #    .
-    CMDDEFAULT => '',
+    CMDDEFAULT => '', # 'void',
     CMD => {
         void => {
             pidcheck => 0, # 0 - OFF, 1 - ON
@@ -232,75 +223,98 @@ use constant {
 use Getopt::Long;
 use Pod::Usage;
 use FindBin qw($RealBin);
-
-# Others:
-# use Data::Dumper;
+use lib "$RealBin/inc";
 
 # CTK Packages
-use lib "$RealBin/inc";
-use base '%PROJECTNAME%';
+use base qw/%PROJECTNAMEUCF%/;
 use CTK;
+use CTKx;
 use CTK::FilePid;
 
-#  
 Getopt::Long::Configure ("bundling");
+GetOptions(\%OPT,
 
-GetOptions(\%OPT, @OPTSYSDEF, # humvdlcyt?
+    #
+    # NoUsed keys map:
+    #
+    # a A b B c C   D e E
+    # f F g G     i I j J
+    # k K   L   M n N o O
+    # p P q Q r R s S   T
+    # u U     w W x X y Y
+    # z Z
+    #
 
-    #   
-    "foo|f=s",            # FOO
-    "bar|b=i",            # BAR
-    "baz|z=s",            # BAZ
-    
-) || pod2usage(-exitval => 1, -verbose => 0, -output => \*CTK::CTKCP);
-pod2usage(-exitval => 0, -verbose => 0, -output => \*CTK::CTKCP) if $OPT{help};
-pod2usage(-exitval => 0, -verbose => 99, -sections => 'NAME|VERSION', -output => \*CTK::CTKCP) if $OPT{version};
-pod2usage(-exitval => 0, -verbose => 2, -output => \*CTK::CTKCP) if $OPT{man};
+    # Help
+    "help|usage|h|?",       # Show short help page
+    "longhelp|H",           # Show long help page
+    "version|ver|V",        # Show VERSION
+
+    # Debugging
+    "debug|d",              # Debug mode
+    "log|l",                # Log mode
+    "verbose|v",            # Verbose mode. Default: Silent mode
+    "logclear|logclean",    # Clear log
+    "signature=s",          # Sign to log
+    "testmode|test|t",      # Testmode
+
+    # Application
+    "foo|f=s",              # FOO
+    "bar|b=i",              # BAR
+    "baz|z=s",              # BAZ
+
+) || pod2usage(-exitval => 1, -verbose => 0, -output => \*STDERR);
+pod2usage(-exitval => 0, -verbose => 1) if $OPT{help};
+pod2usage(-exitval => 0, -verbose => 2) if $OPT{longhelp};
+say(%PROJECTNAMEUCF%->VERSION) && exit(0) if $OPT{version};
 
 # VARS
 my %cmddata;
 
-# 
-my $command   = @ARGV ? shift @ARGV : CMDDEFAULT; # 
-my @arguments = @ARGV ? @ARGV : (); #  
-my @commands  = keys %{sub{CMD}->()}; # @{sub{COMMANDS}->()}
-pod2usage(-exitval => 1, -verbose => 99, -sections => 'SYNOPSIS|OPTIONS|COMMANDS', -output => \*CTK::CTKCP)
+# Commands
+my $command   = @ARGV ? shift @ARGV : CMDDEFAULT;
+my @arguments = @ARGV ? @ARGV : ();
+my @commands  = keys %{sub{CMD}->()};
+pod2usage(-exitval => 1, -verbose => 99, -sections => 'SYNOPSIS|OPTIONS|COMMANDS', -output => \*STDERR)
     if ( (grep {$_ eq $command} @commands) ? 0 : 1 );
-    
-# Preparing directories and Log Clear 
+
+# Preparing directories and Log Clear
 CTK::preparedir({
         logdir  => $LOGDIR,
         datadir => $DATADIR,
     });
+
+my $ctkx = CTKx->instance( c => new CTK( logdir => CTK::catdir($RealBin, CTK::LOGDIRD()) ) );
 unlink( $LOGFILE ) if( $OPT{logclear} && -e $LOGFILE ); # Remove unnesessary log
 
 START:  debug "-"x16, " START ", (testmode() ? 'IN TEST MODE ' : ''), tms," ","-"x16;
 #########################
 ### START
 #########################
-
-my $code = __PACKAGE__->can(uc($command));
+my $pidfile;
+my $code = __PACKAGE__->can(uc($command)) || __PACKAGE__->can($command);
 if ($code && ref($code) eq 'CODE') {
     %cmddata = %{CMD->{$command}};
     $cmddata{arguments} = [@arguments];
 
-    #  PID    
-    my $pidfile = new CTK::FilePid({ file => CTK::catfile($DATADIR, $cmddata{pidfile} || PIDFILE) });
+    # PID File define
+    $pidfile = new CTK::FilePid({ file => CTK::catfile($DATADIR, $cmddata{pidfile} || PIDFILE) });
     my $pidstat = $pidfile->running || 0;
-    
+
     debug "==== START COMMAND: ".uc($command)." ($$) ====";
-    
+
     if ($cmddata{pidcheck}) {
         exception("PID STATE (".$pidfile->file()."): ALREADY EXISTS (PID: $pidstat)" ) if $pidstat;
         $pidfile->write;
     }
 
-    &{$code}(%cmddata); #       
-    
+
+    &{$code}(%cmddata);
+
     if ($cmddata{pidcheck}) {
         $pidfile->remove;
     }
-    
+
     debug "==== FINISH COMMAND: ".uc($command)." ($$) ====";
 } else {
     exception("Sub \"".uc($command)."\" undefined");
@@ -314,51 +328,52 @@ exit(0);
 
 1;
 __END__
-
-NoUsed keys map:
-
-a A b B   C   D e E 
-f F g G   H i I j J 
-k K   L   M n N o O 
-p P q Q r R s S   T 
-  U   V w W x X   Y 
-z Z
 -----END FILE-----
 
 -----BEGIN FILE-----
-Name: %PROJECTNAME%.pm
-File: %PROJECTNAME%/inc/%PROJECTNAME%.pm
+Name: %PROJECTNAMEUCF%.pm
+File: %PROJECTNAME%/inc/%PROJECTNAMEUCF%.pm
 Mode: 644
 
-package %PROJECTNAME%; # %DOLLAR%Id%DOLLAR%
+package %PROJECTNAMEUCF%; # %DOLLAR%Id%DOLLAR%
 use strict;
 
 use vars qw/$VERSION/;
-$VERSION = 1.00;
+$VERSION = '1.00';
 
+use CTKx;
 use CTK qw/:BASE/;
-#use CTK::DBI;
+use Data::Dumper; $Data::Dumper::Deparse = 1;
 
-sub VOID {
-    #  
+sub void {
     debug("VOID CONTEXT");
     1;
 }
-sub TEST {
-    #  
-    debug(" ");
-    my %cmd = @_; #debug(join "; ",@{$cmd{arguments}});
+sub test {
+    debug("Testing...");
+    debug;
+
+    my $c = CTKx->instance->c;
+    my %cmd = @_;
+
+    my @arguments = @{$cmd{arguments}};
+
+    debug("Argumens: ", join("; ",@arguments) || 'none' );
+
+    debug("Input params:");
+    debug(Dumper(\%cmd));
+
+    # CTK debug
+    my $config = $c->config;
+    if (verbosemode) {
+        debug("CTK object:");
+        debug(Dumper($c));
+    }
 
-    #   
-    my $c = new CTK;
-    
-    #my $config = $c->config;
-    #debug(Dumper($config));
-    
     #
-    # ...
+    # . . .
     #
-    
+
     1;
 }
 
@@ -367,14 +382,14 @@ __END__
 -----END FILE-----
 
 -----BEGIN FILE-----
-Name: %PROJECTNAME%.log
-File: %PROJECTNAME%/log/%PROJECTNAME%.log
+Name: %PROJECTNAMEL%.log
+File: %PROJECTNAME%/log/%PROJECTNAMEL%.log
 Mode: 666
 -----END FILE-----
 
 -----BEGIN FILE-----
-Name: %PROJECTNAME%.conf
-File: %PROJECTNAME%/%PROJECTNAME%.conf
+Name: %PROJECTNAMEL%.conf
+File: %PROJECTNAME%/%PROJECTNAMEL%.conf
 Mode: 666
 
 ################################################################################
@@ -386,6 +401,7 @@ Mode: 666
 ################################################################################
 
 Include conf/*.conf
+
 -----END FILE-----
 
 -----BEGIN FILE-----
@@ -395,47 +411,44 @@ Mode: 666
 
 ################################################################################
 #
-#     sendmail     
+# SendMail section
 #
 ################################################################################
 <SendMail>
-    #    
     to          to@example.com
     cc          cc@example.com
     from        from@example.com
-    
-    #      
+
     testmail    test@example.com
     errormail   error@example.com
-    
-    #         UTF-8
+
+    # Mail source encoding for to UTF-8 converting
     charset     windows-1251
     type        text/plain
-    
-    #  sendmail   
+
+    # sendmail and arguments
     sendmail    /usr/sbin/sendmail
     flags       -t
-    
-    # SMTP ,  . 
-    # SMTP      sendmail
+
+    # SMTP server address
     smtp        192.168.1.1
-    
-    #  SMTP
+
+    # Authorization SMTP data (optional)
     #smtpuser user
     #smtppass password
 
-    #   (    )
+    # Attach files
     #<Attach>
-    #    Filename    doc1.txt 
-    #    Type        text/plain 
+    #    Filename    doc1.txt
+    #    Type        text/plain
     #    Disposition attachment
     #    Data        "Document 1. Content"
     #</Attach>
     #<Attach>
-    #    Filename    mbutiny.log
+    #    Filename    %PROJECTNAMEL%.log
     #    Type        text/plain
     #    Disposition attachment
-    #    Path        /var/log/mbutiny.log
+    #    Path        log/%PROJECTNAMEL%.log
     #</Attach>
 </SendMail>
 -----END FILE-----
@@ -447,7 +460,7 @@ Mode: 666
 
 ################################################################################
 #
-#     Oracle/MySQL/SQLite/PGSQL
+# DBI database section: Oracle/MySQL/SQLite/PGSQL and etc.
 #
 ################################################################################
 
@@ -471,40 +484,7 @@ Mode: 666
 
 ################################################################################
 #
-#    .  .   CTK
-# 
-#         ,
-#        
-# .       :
-#
-#      :
-#    FILE     --     
-#    FILENAME --    
-#    DIRSRC   --    
-#    DIRIN    -- = DIRSRC
-#    DIRDST   --     
-#    DIROUT   -- = DIRDST
-#    LIST     --    ,  
-#    EXC      -- 'exclude file' !!!!!!
-#
-#        :
-#    FILE     --       
-#    DIRSRC   --        
-#    DIRIN    -- = DIRSRC
-#    LIST     --    ,  
-#    EXC      -- 'exclude file' !!!!!!
-#
-#        tar
-# 
-# <Arc tgz> #   . ,  ,    
-#    type       tar                       #  ,   
-#    ext        tgz                       #   
-#    create     tar -zcpf [FILE] [LIST]   #    
-#    extract    tar -zxpf [FILE] [DIRDST] #      
-#    exclude    --exclude-from            # !!!!!!
-#    list       tar -ztf [FILE]           #       
-#    nocompress tar -cpf [FILE]           #      
-# </Arc>
+# Archivation section
 #
 ################################################################################
 
@@ -556,9 +536,9 @@ Mode: 666
     nocompress zip -qq0
 </Arc>
 
-# bzip2 
+# bzip2
 <Arc bz2>
-    type       bzip2 
+    type       bzip2
     ext        bz2
     create     bzip2 --best [FILE] [LIST]
     extract    bzip2 -d [FILE]
@@ -587,22 +567,13 @@ File: %PROJECTNAME%/README
 Mode: 644
 
 NAME
-    %PROJECTNAME%.pl - My application
+    %PROJECTNAMEL% - My application
 
 VERSION
     Version 1.00
 
 SYNOPSIS
-    %PROJECTNAME%.pl [options] [commands [args]] 
-
-    %PROJECTNAME%.pl [-lcdt]
-
-    %PROJECTNAME%.pl [-h | -v | -m]
-
-    %PROJECTNAME%.pl [--help | --version | --man]
-    
-    %PROJECTNAME%.pl [--log] [--logclear] [--debug] [--testmode] [--signature=MESSAGE]
-           [ test | void ]
+    %PROJECTNAMEL% [options] [commands [args]]
 
 ABSTRACT
     The project ...
@@ -632,10 +603,10 @@ THANKS
     Thanks to me for ...
 
 AUTHOR
-    Your Name (Your Nickname) http://your.site <your@email.com>
+    Your Name (Your Nickname) http://www.your.site <your@email.com>
 
 COPYRIGHTS
-    Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
+    Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 LICENSE
     This program is distributed under the GNU GPL v3.
@@ -703,29 +674,6 @@ Mode: 644
 
 -----END FILE-----
 
------BEGIN FILE-----
-Name: MANIFEST
-File: %PROJECTNAME%/MANIFEST
-Mode: 644
-
-# Generated by CTKlib %CTKVERSION%
-# %DOLLAR%Id%DOLLAR%
-
-%PROJECTNAME%.conf
-conf/extra-arc.conf             Archivator settings
-conf/extra-store.conf           Store settings
-conf/extra-sendmail.conf        Sendmail settings
-inc/%PROJECTNAME%.pm
-%PROJECTNAME%.pl
-CHANGES                         Changes list
-INSTALL                         Installation instructions
-LICENSE                         License file
-Makefile.PL                     MAkefile builder
-MANIFEST                        This file
-README                          !!! README FIRST !!!
-TODO                            TO DO
------END FILE-----
-
 -----BEGIN FILE-----
 Name: LICENSE
 File: %PROJECTNAME%/LICENSE
