@@ -1,4 +1,4 @@
-package CTK::Log; # $Id: Log.pm 180 2014-04-14 19:59:32Z minus $
+package CTK::Log; # $Id: Log.pm 193 2017-04-29 07:30:55Z minus $
 use Moose::Role; # use Data::Dumper; $Data::Dumper::Deparse = 1;
 use Moose::Util::TypeConstraints;
 
@@ -8,11 +8,7 @@ CTK::Log - CTK Logging methods
 
 =head1 VERSION
 
-Version 2.59
-
-=head1 REVISION
-
-$Revision: 180 $
+Version 2.60
 
 =head1 SYNOPSIS
 
@@ -21,31 +17,117 @@ $Revision: 180 $
             logfile      => CTK::catfile($LOGDIR,'foo.log'),
             logseparator => " ", # as default
         );
-        
+
     $c->log( INFO => " ... Blah-Blah-Blah ... " );
 
-    $c->log_except()  # 9 exception
-    $c->log_fatal()   # 8 fatal
-    $c->log_emerg()   # 7 system is unusable
-    $c->log_alert()   # 6 action must be taken immediately
-    $c->log_crit()    # 5 critical conditions
-    $c->log_error()   # 4 error conditions
-    $c->log_warning() # 3 warning conditions
-    $c->log_notice()  # 2 normal but significant condition
-    $c->log_info()    # 1 informational
-    $c->log_debug()   # 0 debug-level messages (default)
-    
+    $c->log_except();  # 9 exception
+    $c->log_fatal();   # 8 fatal
+    $c->log_emerg();   # 7 system is unusable
+    $c->log_alert();   # 6 action must be taken immediately
+    $c->log_crit();    # 5 critical conditions
+    $c->log_error();   # 4 error conditions
+    $c->log_warning(); # 3 warning conditions
+    $c->log_notice();  # 2 normal but significant condition
+    $c->log_info();    # 1 informational
+    $c->log_debug();   # 0 debug-level messages (default)
+
 =head1 DESCRIPTION
 
 All of methods are returned by log-records
 
+    # Log File defined in constructor
+    $c = new CTK (
+            loglevel     => 'info', # or '1'
+            logfile      => CTK::catfile($LOGDIR,'foo.log'),
+            logseparator => " ", # as default
+        );
+
+    # Log File defined after constructor
+    $c->logfile(catfile($c->logdir(),
+            dformat($cmddata{logfile}, {
+                        COMMAND  => $command,
+                        PREFIX   => $c->prefix(),
+                        SUFFIX   => $c->suffix(),
+                        EXT      => 'log',
+                        DEFAULT  => LOGFILE,
+                    }
+                )
+            )
+        );
+
+=head2 log
+
+    $c->log( INFO => " ... Blah-Blah-Blah ... " );
+
+Logging with info level (1). Same as log_info
+
+=head2 log_debug, logdebug, logdebugging
+
+    $c->log_debug();
+
+Level 0: debug-level messages (default)
+
+=head2 log_info, info, loginf, loginfo, loginformation
+
+    $c->log_info();
+
+Level 1: informational
+
+=head2 log_notice, notice, lognote, lognotice
+
+    $c->log_notice();
+
+Level 2: normal but significant condition
+
+=head2 log_warning, warning, logwarn, logwarning
+
+    $c->log_warning();
+
+Level 3: warning conditions
+
+=head2 log_error, error, logerror
+
+    $c->log_error();
+
+Level 4: error conditions
+
+=head2 log_crit, crit, logcrit, logcritical
+
+    $c->log_crit();
+
+Level 5: critical conditions
+
+=head2 log_alert, alert, logalert
+
+    $c->log_alert();
+
+Level 6: action must be taken immediately
+
+=head2 log_emerg, emerg, logemerg, logemergency
+
+    $c->log_emerg();
+
+Level 7: system is unusable
+
+=head2 log_fatal, fatal, logfatal
+
+    $c->log_fatal();
+
+Level 8: fatal
+
+=head2 log_except, except, logexcept, logexception
+
+    $c->log_except();
+
+Level 9: exception
+
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2014 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -58,7 +140,7 @@ See C<LICENSE> file
 =cut
 
 use constant {
-    LOGLEVELS       => { 
+    LOGLEVELS       => {
         'debug'   => 0,
         'info'    => 1,
         'notice'  => 2,
@@ -72,10 +154,8 @@ use constant {
     },
 };
 
-#use Data::Dumper;
-
 use vars qw/$VERSION/;
-$VERSION = '2.59';
+$VERSION = '2.60';
 
 subtype 'LogLevels'
     => as 'Int'
@@ -87,8 +167,8 @@ coerce 'LogLevels'
     => via { ($_ && LOGLEVELS->{$_}) ? LOGLEVELS->{$_} : 0 };
 
 has 'loglevel' => (
-    is         => 'rw', 
-    isa        => 'LogLevels', 
+    is         => 'rw',
+    isa        => 'LogLevels',
     default    => 0,
     lazy       => 1,
     coerce     => 1,
@@ -113,8 +193,8 @@ subtype 'LogSeparators'
     => message { "The logseparator not valid" };
 
 has 'logseparator' => (
-    is         => 'rw', 
-    isa        => 'LogSeparators', 
+    is         => 'rw',
+    isa        => 'LogSeparators',
     default    => ' ',
     lazy       => 1,
 );
@@ -159,7 +239,7 @@ around BUILDARGS => sub {
                 $p->{handle} = $p->{logfile};
             }
         }
-        
+
     }
     return $class->$orig(@_);
 
@@ -168,7 +248,7 @@ around BUILDARGS => sub {
 after DEMOLISH => sub {
     my $self = shift;
     #CTK::debug("DEMOLISH called");
-    
+
     if ($self->sethandle()) {
         my $fh = $self->handle;
         $fh->close() if $fh;
@@ -180,11 +260,11 @@ sub log {
     my $self  = shift;
     my $level = shift;
     my @l = @_;
-    
+
     my $loglevels = LOGLEVELS;
     my %levels  = %$loglevels;
     my %rlevels = reverse %$loglevels;
-    
+
     my $proc = 'log_debug'; #   
     if (defined($level) && ($level =~ /^[0-9]+$/) && defined $rlevels{$level}) {
         $proc = 'log_'.$rlevels{$level};
@@ -196,7 +276,7 @@ sub log {
         unshift @l, $level if defined $level;
         #CTK::debug (@l);
     }
-    
+
     #     
     confess "Undefinned the LogLevel!" unless $proc;
     my $lcode = __PACKAGE__->can("$proc");
@@ -217,63 +297,63 @@ sub info {
     my $l = "info";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_info { info @_ } 
-sub loginfo { info @_ } 
-sub loginformation { info @_ } 
-sub loginf { info @_ } 
+sub log_info { info @_ }
+sub loginfo { info @_ }
+sub loginformation { info @_ }
+sub loginf { info @_ }
 sub notice {
     my $l = "notice";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_notice { notice @_ } 
-sub lognotice { notice @_ } 
-sub lognote { notice @_ } 
+sub log_notice { notice @_ }
+sub lognotice { notice @_ }
+sub lognote { notice @_ }
 sub warning {
     my $l = "warning";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_warning { warning @_ } 
-sub logwarning { warning @_ } 
-sub logwarn { warning @_ } 
+sub log_warning { warning @_ }
+sub logwarning { warning @_ }
+sub logwarn { warning @_ }
 sub error {
     my $l = "error";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_error { error @_ } 
-sub logerror { error @_ } 
+sub log_error { error @_ }
+sub logerror { error @_ }
 sub crit {
     my $l = "crit";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_crit { crit @_ } 
-sub logcrit { crit @_ } 
-sub logcritical { crit @_ } 
+sub log_crit { crit @_ }
+sub logcrit { crit @_ }
+sub logcritical { crit @_ }
 sub alert {
     my $l = "alert";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_alert { alert @_ } 
-sub logalert { alert @_ } 
+sub log_alert { alert @_ }
+sub logalert { alert @_ }
 sub emerg {
     my $l = "emerg";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_emerg { emerg @_ } 
-sub logemerg { emerg @_ } 
-sub logemergency { emerg @_ } 
+sub log_emerg { emerg @_ }
+sub logemerg { emerg @_ }
+sub logemergency { emerg @_ }
 sub fatal {
     my $l = "fatal";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_fatal { fatal @_ } 
-sub logfatal { fatal @_ } 
+sub log_fatal { fatal @_ }
+sub logfatal { fatal @_ }
 sub except {
     my $l = "except";
     _flush(LOGLEVELS->{$l}, $l, @_);
 }
-sub log_except { except @_ } 
-sub logexcept { except @_ } 
-sub logexception { except @_ } 
+sub log_except { except @_ }
+sub logexcept { except @_ }
+sub logexception { except @_ }
 
 sub _flush {
     #    
@@ -289,7 +369,7 @@ sub _flush {
     push @buffer, "[".scalar(localtime(time()))."]";
     push @buffer, "[$level]" if defined $level;
     push @buffer, "[$$]";
-    
+
     push @buffer, @_;
     if ($self->sethandle()) {
         my $fh = $self->handle();
@@ -298,7 +378,5 @@ sub _flush {
     return join($self->logseparator(), @buffer);
 }
 
-#no Moose;
-#__PACKAGE__->meta->make_immutable;
 1;
 __END__
