@@ -1,4 +1,4 @@
-package CTK::Net; # $Id: Net.pm 180 2014-04-14 19:59:32Z minus $
+package CTK::Net; # $Id: Net.pm 193 2017-04-29 07:30:55Z minus $
 use Moose::Role; # use Data::Dumper; $Data::Dumper::Deparse = 1;
 
 =head1 NAME
@@ -7,11 +7,7 @@ CTK::Net - Network working
 
 =head1 VERSION
 
-Version 2.52
-
-=head1 REVISION
-
-$Revision: 180 $
+Version 1.53
 
 =head1 SYNOPSIS
 
@@ -22,7 +18,7 @@ $Revision: 180 $
         ftpdir      => '~/dir01',
         #ftpattr     => {},
     );
-    
+
     my %uaopt = (
         agent                   => "Mozilla/4.0",
         max_redirect            => 10,
@@ -30,7 +26,7 @@ $Revision: 180 $
         keep_alive              => 1,
         env_proxy               => 1,
     );
-    
+
     my %httpct = (
         method     => 'GET',
         url        => 'http://example.com',
@@ -38,13 +34,13 @@ $Revision: 180 $
         #password   => 'password',
         #utf8       => 1,
     );
-    
+
     $c->fetch(
             -connect  => {%ftpct},   # Connect data
             -protocol => 'ftp',      # Protocol: ftp / sftp
             -dir      => $DATADIR,   # Destination directory
             -cmd      => 'copyuniq', # Command: copy / copyuniq / move / moveuniq
-            -mode     => 'bin',      # Transfer mode: ascii / binary (bin) 
+            -mode     => 'bin',      # Transfer mode: ascii / binary (bin)
             -list     => qr//, # Source mask (regular expression, filename or ArrayRef of files)
         );
 
@@ -58,8 +54,8 @@ $Revision: 180 $
             #-reqcode => sub { },    # Handler (code) of HTTP::Request (optional)
             #-rescode => sub { },    # Handler (code) of HTTP::Response (optional)
         );
-    debug("STATUS: ",$stat); 
-    
+    debug("STATUS: ",$stat);
+
     # Simple sample
     my $html = $c->fetch(
             -connect  => {
@@ -70,37 +66,65 @@ $Revision: 180 $
             #-utf8     => 1,
         );
     debug("DATA:\n\n",$html,"\n");
-    
+
     $c->store(
             -connect  => {%ftpct},   # Connect data
             -protocol => 'ftp',      # Protocol: ftp / sftp
             -dir      => $DATADIR,   # Source directory
             -cmd      => 'copyuniq', # Command: copy / copyuniq / move / moveuniq
-            -mode     => 'bin',      # Transfer mode: ascii / binary (bin) 
+            -mode     => 'bin',      # Transfer mode: ascii / binary (bin)
             -file     => 'sample.t', # Source mask (regular expression, filename or ArrayRef of files)
         );
 
 =head1 DESCRIPTION
 
-Using handlers, for sample: 
+Using handlers, for sample:
 
     -rescode => sub { debug(CTK::Net::_debug_http(@_)) },
 
     or
-    
+
     -rescode => sub { debug($c->debug_http(@_)) },
-    
+
+=head2 fetch, get, download
+
+    my $stat = $c->fetch(
+            -connect  => {%httpct},  # Connect data
+            -uaopt    => {%uaopt},   # Options UserAgent (optional)
+            -protocol => 'http',     # Protocol: http / https
+            -dir      => $DATADIR,   # Destination directory
+            #-file    => '123.html', # Filename (optional)
+            #-uacode  => sub { },    # Handler (code) of LWP::UserAgent (optional)
+            #-reqcode => sub { },    # Handler (code) of HTTP::Request (optional)
+            #-rescode => sub { },    # Handler (code) of HTTP::Response (optional)
+        );
+
+Download content from resource
+
+=head2 store, put, upload
+
+    $c->store(
+            -connect  => {%ftpct},   # Connect data
+            -protocol => 'ftp',      # Protocol: ftp / sftp
+            -dir      => $DATADIR,   # Source directory
+            -cmd      => 'copyuniq', # Command: copy / copyuniq / move / moveuniq
+            -mode     => 'bin',      # Transfer mode: ascii / binary (bin)
+            -file     => 'sample.t', # Source mask (regular expression, filename or ArrayRef of files)
+        );
+
+Upload content to resource
+
 =head1 TO DO
 
     * Use SSH (SFTP)
 
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2014 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -113,7 +137,7 @@ See C<LICENSE> file
 =cut
 
 use vars qw/$VERSION/;
-$VERSION = '1.52';
+$VERSION = '1.53';
 
 use Encode;
 use CTK::Util qw(:API :FORMAT :ATOM :FILE);
@@ -132,12 +156,12 @@ sub fetch { # fetch (get, download)
     # move     -    ()
     # moveuniq -      
     my $self; $self = shift if (@_ && $_[0] && ref($_[0]) eq 'CTK');
-    
+
     my @args = @_;
-    my ($protocol, $connect, $command, $listmsk, $dirdst, $mode, 
+    my ($protocol, $connect, $command, $listmsk, $dirdst, $mode,
         $uaopt,$uacode, $reqcode, $rescode);
-       ($protocol, $connect, $command, $listmsk, $dirdst, $mode, 
-        $uaopt, $uacode, $reqcode, $rescode) = 
+       ($protocol, $connect, $command, $listmsk, $dirdst, $mode,
+        $uaopt, $uacode, $reqcode, $rescode) =
             read_attributes([
                 ['PROTOCOL','PROTO'],
                 ['CNT','CONNECT','CT'],
@@ -151,7 +175,7 @@ sub fetch { # fetch (get, download)
                 ['RESCODE'],
 
             ],@args) if defined $args[0];
-    
+
     $protocol ||= '';     # : ftp/http/https
     $connect  ||= {};     #  
     $command  ||= 'copy'; # : copy / copyuniq / move / moveuniq
@@ -159,7 +183,7 @@ sub fetch { # fetch (get, download)
     $dirdst   ||= '';     # -
     $mode     ||= '';     #  : none / ascii / binary (bin)
     $uaopt    ||= {};     #   LWP::UserAgent
-    
+
     my $list;
     if ($protocol eq 'ftp') {
         if (ref($listmsk) eq 'ARRAY') {
@@ -173,23 +197,23 @@ sub fetch { # fetch (get, download)
             $list = ftpgetlist($connect,qr/$listmsk/);
         }
     }
-    
+
     if ($protocol eq 'ftp') {
         #CTK::debug("Get files from ftp://$connect->{ftphost}...");
         my $ftph = ftp($connect, 'connect');
-        
+
         my $i = 0;
         my $c = scalar(@$list) || 0;
         foreach my $fn (@$list) {$i++;
             my $fs = $ftph->size($fn) || 0;
             #CTK::debug("   Get file $i/$c $fn [".correct_number($fs)." b]...");
-            
+
             my $fndst = catfile($dirdst,$fn);
-            
+
             $ftph->binary if $mode eq 'binary';
             $ftph->binary if $mode eq 'bin';
             $ftph->ascii  if $mode eq 'ascii';
-            
+
             my $statget = 0;
             if (($command =~ /uniq/) && (-e $fndst) && (-s $fndst) == $fs) {
                 #   ,    
@@ -198,19 +222,19 @@ sub fetch { # fetch (get, download)
             } else {
                 $statget = $ftph->get($fn,$fndst);
             }
-            
+
             my $fsdst = $statget && -e $fndst ? (-s $fndst) : 0; #   
             if ($statget && $fsdst >= $fs) {
                 #  
                 if ($command =~ /move/) {
                     # ,     
                     #CTK::debug("   Deleting file $i/$c $fn...");
-                    $ftph->delete($fn) or 
+                    $ftph->delete($fn) or
                         _error( "FETCHING FTP ERROR: Can't delete file \"$fn\": ", $ftph->message );
                 }
             } else {
                 if ($statget) {
-                    _error("FETCHING FTP ERROR: Can't get file \"$fn\": ", $ftph->message); 
+                    _error("FETCHING FTP ERROR: Can't get file \"$fn\": ", $ftph->message);
                 } else {
                     _error("FETCHING FTP ERROR: File size \"$fn\" ($fs) < \"$fndst\" ($fsdst) ");
                 }
@@ -218,27 +242,27 @@ sub fetch { # fetch (get, download)
         }
         $ftph->quit();
         return 1;
-        
+
     } elsif ($protocol =~ /^https?$/) {
         CTK::exception("Param UAOPT icorrect") if ref($uaopt) ne 'HASH';
-        my $ua  = new LWP::UserAgent(%$uaopt); 
+        my $ua  = new LWP::UserAgent(%$uaopt);
         $uacode->($ua) if ($uacode && ref($uacode) eq 'CODE');
-        
+
         #    
         my $method   = $connect->{method} || 'GET';
         my $url      = $connect->{url} || '';
         $url         = new URI($url);
-        
+
         my $login    = defined($connect->{login}) ? $connect->{login} : '';
         my $password = defined($connect->{password}) ? $connect->{password} : '';
         my $onutf8   = $connect->{'utf8'} || 0;
-        
+
         my $req = new HTTP::Request(uc($method), $url);
         $req->authorization_basic($login, $password) if defined($connect->{login});
         $reqcode->($req) if ($reqcode && ref($reqcode) eq 'CODE');
         my $res = $ua->request($req);
         $rescode->($res) if ($rescode && ref($rescode) eq 'CODE');
-    
+
         my $html = '';
         if ($res->is_success) {
             if ($onutf8) {
@@ -252,10 +276,10 @@ sub fetch { # fetch (get, download)
         } else {
             _error("FETCHING HTTP ERROR: An error occurred while trying to obtain the resource \"$url\" (",$res->status_line,")");
         }
-        
+
         #      
         # $dirdst  - 
-        # $listmsk -    
+        # $listmsk -  
         my $file;
         if ($dirdst || $listmsk) {
             #     
@@ -284,16 +308,16 @@ sub fetch { # fetch (get, download)
                 }
             }
             $file = catfile($dirdst,$file) if $dirdst && -e $dirdst; #  
-            
+
             #       
             bsave($file, $html, $onutf8);
             return $res->is_success ? 1 : 0;
-            
-        } else { 
+
+        } else {
             #    
             return $html;
         }
-        
+
         return 1;
     }
 }
@@ -302,10 +326,10 @@ sub download { fetch(@_) }
 sub store { # store (put, upload)
     #     
     my $self; $self = shift if (@_ && $_[0] && ref($_[0]) eq 'CTK');
-    
+
     my @args = @_;
     my ($protocol, $connect, $command, $listmsk, $dirsrc, $mode);
-       ($protocol, $connect, $command, $listmsk, $dirsrc, $mode) = 
+       ($protocol, $connect, $command, $listmsk, $dirsrc, $mode) =
             read_attributes([
                 ['PROTOCOL','PROTO'],
                 ['CNT','CONNECT','CT'],
@@ -314,8 +338,8 @@ sub store { # store (put, upload)
                 ['SOURCE','DIR','DIRSRC','SRC'],
                 ['MODE','MD']
             ],@args) if defined $args[0];
-    
-    
+
+
     $protocol ||= '';     # 
     $connect  ||= {};     #  
     $command  ||= 'copy'; # : copy / copyuniq / move / moveuniq
@@ -338,7 +362,7 @@ sub store { # store (put, upload)
     if ($protocol eq 'ftp') {
         #CTK::debug("Store files to ftp://$connect->{ftphost}...");
         my $ftph = ftp($connect,'connect');
-        
+
         my $i = 0;
         my $c = scalar(@$list) || 0;
         foreach my $fn (@$list) {$i++;
@@ -359,19 +383,19 @@ sub store { # store (put, upload)
             } else {
                 $statput = $ftph->put($fsrc,$fn);
             }
-            
+
             my $fsdst = $ftph->size($fn) || 0; #   
             if ($statput && $fsdst >= $fs) {
                 #  
                 if ($command eq 'move') {
                     # ,     
                     #CTK::debug("   Deleting file $i/$c $fn...");
-                    unlink($fsrc) or 
+                    unlink($fsrc) or
                         _error( "STORING FTP ERROR: Cannot delete file \"$fn\": $!");
                 }
             } else {
                 if ($statput) {
-                    _error("STORING FTP ERROR: Cannot put file \"$fn\": ", $ftph->message); 
+                    _error("STORING FTP ERROR: Cannot put file \"$fn\": ", $ftph->message);
                 } else {
                     _error("STORING FTP ERROR: File size \"$fn\" ($fsdst) < \"$fsrc\" ($fs) ");
                 }
@@ -379,9 +403,9 @@ sub store { # store (put, upload)
         }
         $ftph->quit();
     }
-    
+
     return 1;
-    
+
 }
 sub put { store(@_) }
 sub upload { store(@_) }
@@ -391,12 +415,12 @@ sub _debug_http {
     my $self; $self = shift if (@_ && $_[0] && ref($_[0]) eq 'CTK');
 
     my $res = shift || return '';
-    
+
     return "\n\nREQUEST-HEADERS:\n\n",
     $res->request->method, " ", $res->request->url->as_string,"\n",
     $res->request->headers_as_string,
     "\n\nREQUEST-CONTENT:\n\n",$res->request->content,
-    "\n\nRESPONSE:\n\n",$res->code," ",$res->message,"\n",$res->headers_as_string;	
+    "\n\nRESPONSE:\n\n",$res->code," ",$res->message,"\n",$res->headers_as_string;
 }
 sub _error {
     #CTK::debug(@_);
