@@ -1,4 +1,4 @@
-package CTK::Util; # $Id: Util.pm 180 2014-04-14 19:59:32Z minus $
+package CTK::Util; # $Id: Util.pm 198 2017-04-30 19:24:58Z minus $
 use strict; # use Data::Dumper; $Data::Dumper::Deparse = 1;
 
 =head1 NAME
@@ -7,26 +7,22 @@ CTK::Util - CTK Utilities
 
 =head1 VERSION
 
-Version 2.75
-
-=head1 REVISION 
-
-$Revision: 180 $
+Version 2.76
 
 =head1 SYNOPSIS
 
     use CTK::Util;
     use CTK::Util qw( :BASE ); # Export only BASE subroutines. See TAGS section
-    
+
     my @ls = ls(".");
-    
+
     # or (for CTK module)
-    
+
     use CTK;
     my @ls = CTK::ls(".");
-    
+
     # or (for core and extended subroutines only)
-    
+
     use CTK;
     my $c = new CTK;
     my $prefix = $c->getsyscfg("prefix");
@@ -78,9 +74,9 @@ See L<File::Spec::Functions> and L<File::Spec> for details
 
     $path = catdir( @directories );
 
-Concatenate two or more directory names to form a complete path ending with a directory. 
-But remove the trailing slash from the resulting string, because it doesn't look good, isn't 
-necessary and confuses OS/2. Of course, if this is the root directory, don't cut off the 
+Concatenate two or more directory names to form a complete path ending with a directory.
+But remove the trailing slash from the resulting string, because it doesn't look good, isn't
+necessary and confuses OS/2. Of course, if this is the root directory, don't cut off the
 trailing slash :-)
 
 =item B<catfile>
@@ -118,20 +114,20 @@ The opposite of "catdir"
     ($volume,$directories,$file) = splitpath( $path );
     ($volume,$directories,$file) = splitpath( $path, $no_file );
 
-Splits a path in to volume, directory, and filename portions. On systems with no concept of volume, 
+Splits a path in to volume, directory, and filename portions. On systems with no concept of volume,
 returns '' for volume.
 
-For systems with no syntax differentiating filenames from directories, assumes that the last file is 
-a path unless $no_file is true or a trailing separator or /. or /.. is present. On Unix, this means 
+For systems with no syntax differentiating filenames from directories, assumes that the last file is
+a path unless $no_file is true or a trailing separator or /. or /.. is present. On Unix, this means
 that $no_file true makes this return ( '', $path, '' ).
 
 =item B<tmpdir>
 
     $tmpdir = tmpdir();
 
-Returns a string representation of the first writable directory from a list of possible temporary 
-directories. Returns the current directory if no writable temporary directories are found. The list 
-of directories checked depends on the platform; e.g. File::Spec::Unix checks $ENV{TMPDIR} (unless 
+Returns a string representation of the first writable directory from a list of possible temporary
+directories. Returns the current directory if no writable temporary directories are found. The list
+of directories checked depends on the platform; e.g. File::Spec::Unix checks $ENV{TMPDIR} (unless
 taint is on) and /tmp.
 
 =item B<updir>
@@ -212,7 +208,7 @@ Tags: BASE, DATE
 
     $time = date2localtime( $date );
 
-Returns time from date format dd.mm.yyyy in time() value in seconds since the system epoch 
+Returns time from date format dd.mm.yyyy in time() value in seconds since the system epoch
 (Midnight, January 1, 1970 GMT on Unix, for example).
 
 See L<Time::Local/"timelocal">
@@ -229,7 +225,7 @@ Tags: BASE, DATE
 
     $time = datetime2localtime( $datetime );
 
-Returns time from datetime format dd.mm.yyyy hh.mm.ss in time() value in seconds since the system epoch 
+Returns time from datetime format dd.mm.yyyy hh.mm.ss in time() value in seconds since the system epoch
 (Midnight, January 1, 1970 GMT on Unix, for example).
 
 See L<Time::Local/"timelocal">
@@ -253,7 +249,7 @@ For example:
 
     # -> 01-foo-bar.baz.tgz
     $string = dformat( "01-[NAME]-bar.[EXT].tgz", {
-                NAME => 'foo', 
+                NAME => 'foo',
                 EXT  => 'baz',
             } );
 
@@ -285,7 +281,7 @@ Tags: BASE, DATE
     $datetime = dtf( $format, $time, 'GMT' ); # TimeZone (Z) = GMT
 
 Returns time in your format.
-Each conversion specification is replaced by appropriate characters as described in the following list. 
+Each conversion specification is replaced by appropriate characters as described in the following list.
 
     s, ss, _s - Seconds
     m, mm, _m - Minutes
@@ -305,40 +301,40 @@ Examples:
 
     # RFC822 (RSS)
     $dt = dtf("%w, %D %MON %YY %hh:%mm:%ss %G", time(), 1); # Tue, 3 Sep 2013 12:31:40 GMT
-    
+
     # RFC850
     $dt = dtf("%W, %DD-%MON-%YY %hh:%mm:%ss %G", time(), 1); # Tuesday, 03-Sep-13 12:38:41 GMT
-    
+
     # RFC1036
     $dt = dtf("%w, %D %MON %YY %hh:%mm:%ss %G", time(), 1); # Tue, 3 Sep 13 12:44:08 GMT
-    
+
     # RFC1123
     $dt = dtf("%w, %D %MON %YYYY %hh:%mm:%ss %G", time(), 1); # Tue, 3 Sep 2013 12:50:42 GMT
-    
+
     # RFC2822
     $dt = dtf("%w, %DD %MON %YYYY %hh:%mm:%ss +0400"); # Tue, 12 Feb 2013 16:07:05 +0400
-    
+
     # W3CDTF, ATOM (Same as RFC 3339/ISO 8601) -- Mail format
     $dt = dtf("%YYYY-%MM-%DDT%hh:%mm:%ss+04:00"); # 2013-02-12T16:10:28+04:00
-    
+
     # CTIME
     $dt = dtf("%w %MON %_D %hh:%mm:%ss %YYYY"); # Tue Feb  2 16:15:18 2013
-    
+
     # CTIME with TimeZone
     $dt = dtf("%w %MON %_D %hh:%mm:%ss %YYYY %Z", time(), 'MSK'); # Tue Feb 12 17:21:50 2013 MSK
-    
+
     # Russian date and time format
     $dt = dtf("%DD.%MM.%YYYY %hh:%mm:%ss"); # 12.02.2013 16:16:53
-    
+
     # DIG form
     $dt = dtf("%YYYY%MM%DD%hh%mm%ss"); # 20130212161844
-    
+
     # HTTP headers format (See CGI::Util::expires)
     $dt = dtf("%w, %DD %MON %YYYY %hh:%mm:%ss %G", time, 1); # Tue, 12 Feb 2013 13:35:04 GMT
-    
+
     # HTTP/cookie format (See CGI::Util::expires)
     $dt = dtf("%w, %DD-%MON-%YYYY %hh:%mm:%ss %G", time, 1); # Tue, 12-Feb-2013 13:35:04 GMT
-    
+
     # COOKIE (RFC2616 as rfc1123-date)
     $dt = dtf("%w, %DD %MON %YYYY %hh:%mm:%ss %G", time, 1); # Tue, 12 Feb 2013 13:35:04 GMT
 
@@ -387,6 +383,20 @@ See also L</"dformat">
 
 Tags: BASE, FORMAT
 
+=head3 file_lf_normalize
+
+    file_lf_normalize( "file.txt" ) or die("Can't normalize file");
+
+Runs L</"file_lf_normalize"> for every string of the file and save result to this file
+
+Tags: BASE, FORMAT
+
+=head3 file_nl_normalize
+
+See L</"file_lf_normalize">
+
+Tags: BASE, FORMAT
+
 =head3 file_load
 
 See L</"bload">
@@ -441,6 +451,14 @@ See L</"getfilelist">
 
 Tags: BASE, FILE, ATOM
 
+=head3 lf_normalize
+
+    my $normalized_string = lf_normalize( $string );
+
+Returns CR/LF normalized string
+
+Tags: BASE, FORMAT
+
 =head3 load_file
 
 See L</"fload">
@@ -472,6 +490,12 @@ A function returns list content of directory $dir by $mask (regexp or scalar str
 
 Tags: BASE, FILE, ATOM
 
+=head3 nl_normalize
+
+See L</"lf_normalize">
+
+Tags: BASE, FORMAT
+
 =head3 preparedir
 
     $status = preparedir( $dir );
@@ -480,9 +504,9 @@ Tags: BASE, FILE, ATOM
     $status = preparedir( $dir, $chmode );
 
 Preparing directory: creation and permission modification.
-The function returns true or false. 
+The function returns true or false.
 
-The $chmode argument should be a octal value, for example: 
+The $chmode argument should be a octal value, for example:
 
     $status = preparedir( [qw/ foo bar baz /], 0777 );
 
@@ -519,7 +543,7 @@ Tags: BASE, FILE, ATOM
 
     @dirs = scandirs( $dir, $mask );
 
-A function returns all directories of directory $dir by $mask (regexp or scalar string) in 
+A function returns all directories of directory $dir by $mask (regexp or scalar string) in
 format: [$path, $name]
 
 Tags: BASE, FILE, ATOM
@@ -528,7 +552,7 @@ Tags: BASE, FILE, ATOM
 
     @files = scanfiles( $dir, $mask );
 
-A function returns all files of directory $dir by $mask (regexp or scalar string) in 
+A function returns all files of directory $dir by $mask (regexp or scalar string) in
 format: [$path, $name]
 
 Tags: BASE, FILE, ATOM
@@ -537,7 +561,7 @@ Tags: BASE, FILE, ATOM
 
     $stat = shred( $file );
 
-Do a more secure overwrite of given files or devices, to make it harder for even very 
+Do a more secure overwrite of given files or devices, to make it harder for even very
 expensive hardware probing to recover the data.
 
 Tags: BASE, FILE, ATOM
@@ -595,7 +619,7 @@ Tags: BASE, DATE
 
     $base64_text = to_base64( $utf8_text );
 
-Function to encode strings into the base64 encoding specified in 
+Function to encode strings into the base64 encoding specified in
 RFC 2045 - I<MIME (Multipurpose Internet
 Mail Extensions)>. The base64 encoding is designed to represent
 arbitrary sequences of octets in a form that need not be humanly
@@ -709,7 +733,7 @@ Executing external (system) command with IPC::Open3 using.
 
 Variables $in, $err and $binmode is OPTIONAL.
 
-$binmode set up binary mode layer as ':raw:utf8' layer (if $binmode is ':raw:utf8', for example) or 
+$binmode set up binary mode layer as ':raw:utf8' layer (if $binmode is ':raw:utf8', for example) or
 regular binary layer (if $binmode is true).
 
 See also L<IPC::Open3>
@@ -726,11 +750,11 @@ Tags: UTIL, ATOM
         voidfile    => './void.txt',
         #ftpattr    => {}, # See Net::FTP
     );
-    
+
     $ftpct  = ftp( \%ftpct, 'connect' ); # Returns the connect's object
     $rfiles = ftp( \%ftpct, 'ls' ); # Returns reference to array of directory listing
     @remotefiles = $rfiles ? grep {!(/^\./)} @$rfiles : ();
-    
+
     ftp( \%ftpct, 'delete', $rfile ); # Delete remote file
     ftp( \%ftpct, 'get', $rfile, $lfile ); # Get remote file to local file
     ftp( \%ftpct, 'put', $lfile, $rfile ); # Put local file to remote file
@@ -755,12 +779,18 @@ Tags: UTIL, ATOM
 
     $status = ftptest( \%ftpct );
 
-FTP connect testing. 
+FTP connect testing.
 
 See L</"ftp">
 
 Tags: UTIL, ATOM
 
+=head3 proccmd, proccommand, procexe, procexec, procrun
+
+Aliases for L</"execute"> command
+
+Tags: UTIL, ATOM
+
 =head3 sendmail
 
     my $sent = sendmail(
@@ -777,28 +807,28 @@ Tags: UTIL, ATOM
         -authuser => '',                   ### OPTIONAL
         -authpass => '',                   ### OPTIONAL
         -attach   => [                     ### OPTIONAL
-            { 
-                Type=>'text/plain', 
-                Data=>'document 1 content', 
-                Filename=>'doc1.txt', 
+            {
+                Type=>'text/plain',
+                Data=>'document 1 content',
+                Filename=>'doc1.txt',
                 Disposition=>'attachment',
             },
             {
-                Type=>'text/plain', 
-                Data=>'document 2 content', 
-                Filename=>'doc2.txt', 
+                Type=>'text/plain',
+                Data=>'document 2 content',
+                Filename=>'doc2.txt',
                 Disposition=>'attachment',
             },
             {
-                Type=>'text/html', 
-                Data=>'blah-blah-blah', 
-                Filename=>'response.htm', 
+                Type=>'text/html',
+                Data=>'blah-blah-blah',
+                Filename=>'response.htm',
                 Disposition=>'attachment',
             },
             {
-                Type=>'image/gif', 
+                Type=>'image/gif',
                 Path=>'aaa000123.gif',
-                Filename=>'logo.gif', 
+                Filename=>'logo.gif',
                 Disposition=>'attachment',
             },
             ### ... ###
@@ -963,7 +993,7 @@ site with i386, Alpha, and PPC platforms might maintain a single /usr/share dire
 centrally-mounted. Note, however, that /usr/share is generally not intended to be shared by different OSes or
 by different releases of the same OS.
 
-Any program or package which contains or requires data that doesnt need to be modified should store that data
+Any program or package which contains or requires data that doesn't need to be modified should store that data
 in /usr/share (or /usr/local/share, if installed locally). It is recommended that a subdirectory be used in
 /usr/share for this purpose.
 
@@ -983,14 +1013,14 @@ For example value can be set as: /var/lib
 
 This hierarchy holds state information pertaining to an application or the system. State information is data that
 programs modify while they run, and that pertains to one specific host. Users must never need to modify files in
-/var/lib to configure a packages operation.
+/var/lib to configure a package's operation.
 
 State information is generally used to preserve the condition of an application (or a group of inter-related
 applications) between invocations and between different instances of the same application. State information
 should generally remain valid after a reboot, should not be logging output, and should not be spooled data.
 
 An application (or a group of inter-related applications) must use a subdirectory of /var/lib for its data. There
-is one required subdirectory, /var/lib/misc, which is intended for state files that dont need a subdirectory;
+is one required subdirectory, /var/lib/misc, which is intended for state files that don't need a subdirectory;
 the other subdirectories should only be present if the application in question is included in the distribution.
 
 /var/lib/<name> is the location that must be used for all distribution packaging support. Different
@@ -1316,6 +1346,8 @@ L</"dtf">,
 L</"eqtime">,
 L</"escape">,
 L</"fformat">,
+L</"file_lf_normalize">,
+L</"file_nl_normalize">,
 L</"file_load">,
 L</"file_save">,
 L</"fload">,
@@ -1326,6 +1358,7 @@ L</"getlist">,
 L</"getsyscfg">,
 L</"isos">,
 L</"isostype">,
+L</"lf_normalize">,
 L</"load_file">,
 L</"localedir">,
 L</"localstatedir">,
@@ -1333,6 +1366,7 @@ L</"localtime2date">,
 L</"localtime2date_time">,
 L</"lockdir">,
 L</"ls">,
+L</"nl_normalize">,
 L</"path">,
 L</"prefixdir">,
 L</"preparedir">,
@@ -1385,6 +1419,10 @@ L</"correct_number">,
 L</"dformat">,
 L</"escape">,
 L</"fformat">,
+L</"file_lf_normalize">,
+L</"file_nl_normalize">,
+L</"lf_normalize">,
+L</"nl_normalize">,
 L</"randchars">,
 L</"randomize">,
 L</"shuffle">,
@@ -1532,11 +1570,11 @@ L<MIME::Lite>, L<CGI::Util>, L<Time::Local>, L<Net::FTP>, L<IPC::Open3>, L<List:
 
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2014 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -1566,12 +1604,12 @@ use constant {
 };
 
 use vars qw/$VERSION/;
-$VERSION = '2.75';
+$VERSION = '2.76';
 
 use Encode;
 use Time::Local;
 use File::Spec::Functions qw/
-        catdir catfile rootdir tmpdir updir curdir 
+        catdir catfile rootdir tmpdir updir curdir
         path splitpath splitdir abs2rel rel2abs
     /;
 use MIME::Base64;
@@ -1603,6 +1641,7 @@ my @est_encoding = qw(
     );
 my @est_format = qw(
         escape unescape slash tag tag_create cdata dformat fformat splitformat
+        lf_normalize nl_normalize file_lf_normalize file_nl_normalize
         correct_number correct_dig
         translate variant_stf randomize randchars shuffle
     );
@@ -1615,9 +1654,9 @@ my @est_file = qw(
         load_file save_file file_load file_save fsave fload bsave bload touch eqtime shred
     );
 my @est_dir = qw(
-        ls scandirs scanfiles getlist getfilelist getdirlist 
+        ls scandirs scanfiles getlist getfilelist getdirlist
         preparedir
-        
+
     );
 my @est_util = qw(
         sendmail send_mail
@@ -1626,7 +1665,7 @@ my @est_util = qw(
     );
 
 our @EXPORT = (
-        @est_core, @est_encoding, @est_format, @est_datetime, 
+        @est_core, @est_encoding, @est_format, @est_datetime,
         @est_file, @est_dir, @est_util,
     );
 our @EXPORT_OK = @EXPORT;
@@ -1639,7 +1678,7 @@ our %EXPORT_TAGS = (
         BASE    => [
                 @est_core,
                 @est_encoding, @est_format,
-                @est_datetime, 
+                @est_datetime,
                 @est_file, @est_dir,
             ],
         FORMAT  => [
@@ -1664,6 +1703,8 @@ our %EXPORT_TAGS = (
 #    !!!
 push @CTK::Util::ISA, qw/CTK::Util::SysConfig/;
 
+my $CRLF = _crlf();
+
 sub to_utf8 {
     #    UTF-8   
     # to_utf8( $string, $charset ) # charset is 'Windows-1251' as default
@@ -1676,7 +1717,7 @@ sub to_utf8 {
 sub to_windows1251 {
     #    UTF-8   
     # to_windows1251( $string, $charset ) # charset is 'Windows-1251' as default
-    
+
     my $ss = shift; # 
     return '' unless defined($ss);
     my $ch = shift || 'Windows-1251'; # 
@@ -1696,10 +1737,10 @@ sub to_base64 {
 
 #
 #  
-# 
+#
 sub slash {
     #
-    #         
+    #        
     #
     my $data_staring = shift;
     return '' unless defined($data_staring);
@@ -1711,8 +1752,8 @@ sub slash {
 }
 sub tag {
     #
-    #         
-    # 
+    #        
+    #
     # <, >, " and ' chars convert to &lt;, &gt;, &quot; and &#39; strings
     #
     my $data_staring = shift;
@@ -1723,7 +1764,7 @@ sub tag {
     $data_staring =~ s/\"/&quot;/g;
     $data_staring =~ s/\'/&#39;/g;
 
-    return $data_staring;    
+    return $data_staring;
 }
 sub tag_create {
     #
@@ -1736,8 +1777,8 @@ sub tag_create {
     $data_staring =~ s/\&lt\;/\</g;
     $data_staring =~ s/\&gt\;/\>/g;
     $data_staring =~ s/\&quot\;/\"/g;
- 
-    return $data_staring;    
+
+    return $data_staring;
 }
 sub cdata {
     my $s = shift;
@@ -1762,6 +1803,37 @@ sub unescape { # Percent-decoding, also known as URL decoding
     $todecode =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
     return $todecode;
 }
+sub lf_normalize {
+    # line feed normalize in string
+    my $s = shift // return '';
+    $s =~ s/(\x0D*\x0A)|(\x0D)/_proxy_crlf($CRLF)/ges;
+    return $s;
+}
+sub nl_normalize { goto &lf_normalize }
+sub file_lf_normalize {
+    # line feed normalize in file. Original: dos2unix
+    my $f = shift // return 0;
+    return 0 unless -e $f;
+    return 0 if -d $f;
+    return 0 unless -w $f;
+    return 0 unless -r $f;
+    return 0 if -B $f;
+
+    local $\;
+
+    my $temp = sprintf('%s.%d.tmp',$f,$$);
+    open ORIG, $f or do { carp "Can't open $f: $!"; return 0 };
+    open TEMP, ">$temp" or do { carp "Can't create $temp: $!"; return 0 };
+    binmode(TEMP);
+    while (my $line = <ORIG>) {
+        print TEMP lf_normalize($line);
+    }
+	close ORIG;
+	close TEMP;
+	rename $temp, $f;
+    return 1;
+}
+sub file_nl_normalize { goto &file_lf_normalize }
 sub dformat { # ,        
     #        
     my $fmt = shift || ''; #   
@@ -1778,7 +1850,7 @@ sub fformat { # ,
     # [FILE]     --      
     my $fmt = shift || ''; # .  : [FILENAME]-blah-blah-blah.[FILEEXT]
     my $fin = shift || ''; #      " ",  void.txt
-    
+
     my ($fn,$fe) = ($fin =~ /^(.+)\.([0-9a-zA-Z]+)$/) ? ($1,$2) : ($fin,'');
     $fmt =~ s/\[FILENAME\]/$fn/ig;
     $fmt =~ s/\[NAME\]/$fn/ig;
@@ -1790,7 +1862,7 @@ sub fformat { # ,
 sub splitformat { goto &fformat }
 #
 #  
-# 
+#
 sub correct_number {
     #    
     my $var = shift || 0;
@@ -1802,7 +1874,7 @@ sub correct_dig {
     #      .         0
     my $dig = shift || '';
     if ($dig =~/^\s*(\d+)\s*$/) {
-        return $1;    
+        return $1;
     }
     return 0;
 }
@@ -1965,16 +2037,16 @@ sub dtf {
     $dth{'%mon'}   = DTF->{MOYS}->[$dt[4] || 0];
     $dth{'%MONTH'} = DTF->{MOY}->[$dt[4] || 0];
     $dth{'%month'} = DTF->{MOY}->[$dt[4] || 0];
-    
+
     # Second block
     $dth2{'%G'}    = 'GMT' if $g;
     $dth2{'%U'}    = 'UTC' if $g;
     $dth2{'%Z'}    = $z;
     $dth2{'%%'}    = '%';
-    
+
     $f =~ s/$_/$dth{$_}/sge for sort { length($b) <=> length($a) } keys %dth;
     $f =~ s/$_/$dth2{$_}/sge for qw/%G %U %Z %%/;
-   
+
     return $f
 }
 sub timef { goto &dtf }
@@ -1983,7 +2055,7 @@ sub datetimef { goto &dtf }
 
 #
 #    
-# 
+#
 sub translate {
     #      ( )
     my $text = shift;
@@ -2093,7 +2165,7 @@ sub load_file {
             read(FILE,$text,-s $filename) unless -z $filename;
             close FILE;
         } else {
-            _error("[FILE TEXT: Can't open file to load] $!");
+            _error("[FILE TEXT: Can't open file to load '$filename'] $!");
         }
     }
     return $text; #  
@@ -2109,9 +2181,9 @@ sub save_file {
         print FILE $text;
         close FILE;
     } else {
-        _error("[FILE TEXT: Can't open file to write] $!");
+        _error("[FILE TEXT: Can't open file to write '$filename'] $!");
     }
-    return 1; #    
+    return 1; #   
 }
 sub fload { goto &load_file } #  
 sub fsave { goto &save_file } #  
@@ -2153,7 +2225,7 @@ sub file_save {
     } else {
         my $ostat = open $OUT, '>', $fn;
         unless ($ostat) {
-            _error("[FILE BIN: Can't open file to write] $!");
+            _error("[FILE BIN: Can't open file to write '$fn'] $!");
             return 0;
         }
         flock $OUT, 2 or _error("[FILE BIN: Can't lock file \'$fn\']");
@@ -2164,7 +2236,7 @@ sub file_save {
     binmode $OUT unless $onutf8;
     print $OUT $content;
     close $OUT if $flc;
-    return 1; #    
+    return 1; #   
 }
 sub bload { goto &file_load } #  
 sub bsave { goto &file_save } #  
@@ -2183,7 +2255,7 @@ sub touch {
         _error("[TOUCH: Can't open file to write] $!");
         return 0;
     }
-        
+
     close $OUT if $ostat;
     utime($t,$t,$fn);
     return 1;
@@ -2208,7 +2280,7 @@ sub preparedir {
     #  ,   ,     0777
     my $din = shift || return 0;
     my $chmod = shift || undef; #0777
-    
+
     my @dirs;
     if (ref($din) eq 'HASH') {
         foreach my $k (values %$din) { push @dirs, $k };
@@ -2223,7 +2295,7 @@ sub preparedir {
         unless ($dir && (-d $dir or -l $dir)) {
             $stat = 0;
             cluck("Directory don't prepare: \"$dir\"");
-        } 
+        }
     }
     return $stat;
 }
@@ -2231,44 +2303,44 @@ sub scandirs {
     #    [,]
     my $dir = shift || cwd() || curdir() || '.'; #   -  
     my $mask = shift || ''; #   -  
-   
+
     my @dirs;
-   
+
     @dirs = grep {!(/^\.+$/) && -d catdir($dir,$_)} ls($dir, $mask);
     @dirs = sort {$a cmp $b} @dirs;
-  
+
     return map {[catdir($dir,$_), $_]} @dirs;
 }
 sub scanfiles {
     #    [,]
     my $dir = shift || cwd() || curdir() || '.'; #   -  
     my $mask = shift || ''; #   -  
-   
+
     my @files;
     @files = grep { -f catfile($dir,$_)} ls($dir, $mask);
     @files = sort {$a cmp $b} @files;
-  
+
     return map {[catfile($dir,$_), $_]} @files;
 }
 sub ls {
     #   
     my $dir = shift || curdir() || '.'; #   -  
     my $mask = shift || ''; #   -  
-    
+
     my @fds;
-    
+
     my $dh = gensym();
     unless (opendir($dh,$dir)) {
         _error("[LS: Can't open directory \"$dir\"] $!");
         return @fds;
     }
-     
+
     @fds = readdir($dh);#      
     closedir($dh);
 
     #      !
     if ($mask && ref($mask) eq 'Regexp') {
-        return grep {$_ =~ $mask} @fds; 
+        return grep {$_ =~ $mask} @fds;
     } else {
         return grep {/$mask/} @fds if $mask;
     }
@@ -2288,18 +2360,18 @@ sub getdirlist {
 #   Util (Atom)
 #
 sub send_mail {
-    # Version 3.01 (with UTF-8 as default character set and attachment support)    
+    # Version 3.01 (with UTF-8 as default character set and attachment support)
     #
     #     MIME::Lite   UTF-8
     #   . 1 - , 0 - .    
     #
-    
+
     my @args = @_;
-    my ($to, $cc, $from, $subject, $message, $type, 
+    my ($to, $cc, $from, $subject, $message, $type,
         $sendmail, $charset, $mailer_flags, $smtp, $smtpuser, $smtppass,$att);
-    
+
     #  
-    ($to, $cc, $from, $subject, $message, $type, 
+    ($to, $cc, $from, $subject, $message, $type,
      $sendmail, $charset, $mailer_flags, $smtp, $smtpuser, $smtppass, $att) =
     read_attributes([
         ['TO','KOMU','ADDRESS'],
@@ -2318,7 +2390,7 @@ sub send_mail {
     ],@args) if defined $args[0];
 
     #     ,   --  
-    $to           ||= ''; 
+    $to           ||= '';
     $cc           ||= '';
     $from         ||= '';
     $subject      ||= '';
@@ -2329,9 +2401,9 @@ sub send_mail {
     $sendmail       = "" if (-e $sendmail) && (-l $sendmail);
     $charset      ||= "Windows-1251";
     $mailer_flags ||= "-t";
-    $smtp         ||= ''; 
-    $smtpuser     ||= ''; 
-    $smtppass     ||= ''; 
+    $smtp         ||= '';
+    $smtpuser     ||= '';
+    $smtppass     ||= '';
     $att          ||= '';
 
     #        
@@ -2339,9 +2411,9 @@ sub send_mail {
         $subject = to_utf8($subject,$charset);
         $message = to_utf8($message,$charset);
     }
-    
+
     #  
-    my $msg = MIME::Lite->new( 
+    my $msg = MIME::Lite->new(
         From     => $from,
         To       => $to,
         Cc       => $cc,
@@ -2353,7 +2425,7 @@ sub send_mail {
     #   
     $msg->attr('content-type.charset' => 'UTF-8');
     $msg->attr('Content-Transfer-Encoding' => 'base64');
-    
+
     #  (  )
     if ($att) {
         if (ref($att) =~ /HASH/i) {
@@ -2377,18 +2449,18 @@ sub send_mail {
         # sendmail    
         $sendstat = $msg->send(sendmail => "$sendmail $mailer_flags");
         _debug("[SENDMAIL: program sendmail not found! \"$sendmail $mailer_flags\"] $!") unless $sendstat;
-                   
+
     } else {
         #   SMTP 
         my %auth;
         %auth = (AuthUser=>$smtpuser, AuthPass=>$smtppass) if $smtpuser;
         eval { $sendstat = $smtp ? $msg->send('smtp',$smtp,%auth) : $msg->send(); };
         _debug("[SENDMAIL: bad send message ($smtp)!] $@") if $@;
-        _debug("[SENDMAIL: bad method send($smtp)!] $!") unless $sendstat; 
+        _debug("[SENDMAIL: bad method send($smtp)!] $!") unless $sendstat;
     }
     #_debug("[SENDMAIL: The mail has been successfully sent to $to ",::tms(),"]") if $sendstat;
 
-    return $sendstat ? 1 : 0;    
+    return $sendstat ? 1 : 0;
 }
 sub sendmail { goto &send_mail }
 sub ftp {
@@ -2409,7 +2481,7 @@ sub ftp {
     my $cmd         = shift || ''; # 
     my $lfile       = shift || ''; #   ( )
     my $rfile       = shift || ''; #   ( )
-    
+
     #   :
     unless ($ftpconnect && (ref($ftpconnect) eq 'HASH') && $ftpconnect->{ftphost}) {
         _exception("Connect's data missing");
@@ -2443,7 +2515,7 @@ sub ftp {
         return $ftp;
     } elsif ( $cmd eq "ls" ){
         #     
-        (my @out = $ftp->ls(WIN ? "" : "-1a" )) 
+        (my @out = $ftp->ls(WIN ? "" : "-1a" ))
             or _debug( "FTP: Can't get directory listing (\"$ftpdir\") from remote FTP server $ftphost: ", $ftp->message );
         $ftp->quit;
         return [@out];
@@ -2452,17 +2524,17 @@ sub ftp {
         _debug("FTP: No filename given as parameter to FTP command $cmd");
     } elsif ($cmd eq "delete") {
         #  
-        $ftp->delete($lfile) 
+        $ftp->delete($lfile)
             or _debug( "FTP: Can't delete file \"$lfile\" on remote FTP server $ftphost: ", $ftp->message );
     } elsif ($cmd eq "get") {
         #  
         $ftp->binary;
-        $ftp->get($rfile,$lfile) 
+        $ftp->get($rfile,$lfile)
             or _debug("FTP: Can't get file \"$lfile\" from remote FTP server $ftphost: ", $ftp->message);
     } elsif ($cmd eq "put") {
         #  
         $ftp->binary;
-        $ftp->put($lfile,$rfile) 
+        $ftp->put($lfile,$rfile)
             or _debug("FTP: Can't put file \"$lfile\" on remote FTP server $ftphost: ", $ftp->message );
     }
 
@@ -2501,13 +2573,13 @@ sub ftpgetlist {
     #        
     my $connect  = shift || {}; #  
     my $mask     = shift || ''; #  
-    
-    my $rfile = ftp($connect, 'ls'); 
+
+    my $rfile = ftp($connect, 'ls');
     my @files = (($rfile && ref($rfile) eq 'ARRAY') ? @$rfile : ());
 
     #      !
     if ($mask && ref($mask) eq 'Regexp') {
-        @files = grep {$_ =~ $mask} @files; 
+        @files = grep {$_ =~ $mask} @files;
     } else {
         @files = grep {/$mask/} @files if $mask;
     }
@@ -2521,10 +2593,10 @@ sub execute {
     my $out  = '';
     my $err  = shift; # !! REFERENCE TO SCALAR
     my $bm   = shift;
-    
+
     my @scmd;
     if ($icmd && ref($icmd) eq 'ARRAY') { @scmd = @$icmd } else { push @scmd, $icmd }
-    
+
     local (*IN, *OUT, *ERR);
     my $pid	= open3(\*IN, \*OUT, \*ERR, @scmd);
 
@@ -2533,27 +2605,27 @@ sub execute {
     binmode(IN, $bm) if defined($bm) && $bm =~ /\:/;
     print IN $in if defined $in;
     close IN;
-    
+
     # 1 Output
     binmode(OUT) if defined($bm) && $bm && $bm =~ /^\d+$/;
     binmode(OUT, $bm) if defined($bm) && $bm =~ /\:/;
     while (<OUT>) { $out .= $_ }
     close OUT;
-    
+
     # 2 Error
     my $ierr = '';
     binmode(ERR) if defined($bm) && $bm && $bm =~ /^\d+$/;
     binmode(ERR, $bm) if defined($bm) && $bm =~ /\:/;
     while (<ERR>) { $ierr .= $_ }
     close ERR;
-    
+
     waitpid($pid, 0);
     if ($err && ref($err) eq 'SCALAR') {
         $$err = $ierr
     } else {
         _debug("Executable error (".join(" ",@scmd)."): $ierr") if $ierr;
     }
-    
+
     return $out;
 }
 sub exe { goto &execute }
@@ -2574,7 +2646,7 @@ sub procrun { goto &execute }
 # prefixdir localstatedir sysconfdir srvdir
 # sharedir docdir localedir cachedir syslogdir spooldir rundir lockdir sharedstatedir webdir
 #
-sub prefixdir { 
+sub prefixdir {
     my $pfx = __PACKAGE__->ext_syscfg('prefix') ;
     return defined $pfx ? $pfx : '';
 }
@@ -2677,9 +2749,23 @@ sub _make_attributes {
     }
     return @att;
 }
+
 sub _debug { goto &carp } #   
 sub _error { goto &carp } #     STDERROR,    !!!
 sub _exception { goto &confess } #     STDERROR  ,    !!!
+
+sub _crlf {
+    # Original: CGI::Simple
+    return "\015\012" if isostype('Windows');
+    return "\012" if isostype('Unix');
+    my $OS = $^O || do { require Config; $Config::Config{'osname'} };
+    return
+        ( $OS =~ m/VMS/i )   ? "\n"
+        : ( "\t" ne "\011" ) ? "\r\n"
+        :                      "\015\012";
+}
+sub _proxy_crlf {shift}
+
 1;
 
 package  # hide me from PAUSE
@@ -2693,7 +2779,7 @@ sub ext_syscfg {
     #    
     my $caller; $caller = shift if (@_ && $_[0] && $_[0] eq 'CTK::Util');
     my $self; $self = shift if (@_ && $_[0] && ref($_[0]) eq 'CTK');
-    
+
     my $param = shift;
     if (defined $param) {
         return $Config::Config{$param}
