@@ -1,4 +1,4 @@
-package CTKx; # $Id: CTKx.pm 155 2013-10-15 09:50:12Z minus $
+package CTKx; # $Id: CTKx.pm 192 2017-04-28 20:40:38Z minus $
 use strict;
 
 =head1 NAME
@@ -7,11 +7,7 @@ CTKx - User extension CTK
 
 =head1 VERSION
 
-Version 1.00
-
-=head1 REVISION
-
-$Revision: 155 $
+Version 1.01
 
 =head1 SYNOPSIS
 
@@ -19,7 +15,7 @@ $Revision: 155 $
 
     use CTK;
     use CTKx;
-    
+
     my $ctkx = CTKx->instance( c => new CTK );
 
     package MyApp;
@@ -34,6 +30,12 @@ CTKx - User extension CTK
 
 Extension for working with CTK as "Singleton Pattern"
 
+=head2 c
+
+    my $c = CTKx->instance->c;
+
+Returns c-object
+
 =head1 HISTORY
 
 =over 8
@@ -68,11 +70,11 @@ The usual warnings if it can't read or write the files involved.
 
 =head1 AUTHOR
 
-Serz Minus (Lepenkov Sergey) L<http://www.serzik.ru> E<lt>minus@mail333.comE<gt>
+Sergey Lepenkov (Serz Minus) L<http://www.serzik.com> E<lt>minus@mail333.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1998-2013 D&D Corporation. All Rights Reserved
+Copyright (C) 1998-2017 D&D Corporation. All Rights Reserved
 
 =head1 LICENSE
 
@@ -86,7 +88,7 @@ See C<LICENSE> file
 
 use base qw/Class::Singleton/;
 use vars qw($VERSION);
-$VERSION = '1.00';
+$VERSION = '1.01';
 
 sub c { shift->{c} }
 
